#ifndef ADCSTATUS_H
#define ADCSTATUS_H

#include <optional>
#include <QMap>
#include <QMetaType>

#include "ModularDeviceStatus.h"
#include "base/ConfigKey.h"
#include "device-modules/OneWire/BMStatus.h"

struct AdcCoreStatus
{
    friend bool operator==(const AdcCoreStatus &, const AdcCoreStatus &) = default;
    QString modeStr() const;

    bool ctrl_bit_run = false;
    bool running = false;
};

class AdcStatus: public ModularDeviceStatus
{
public:
    explicit AdcStatus(const DeviceIndex &deviceIndex = {})
        : ModularDeviceStatus(deviceIndex) {}

    friend bool operator==(const AdcStatus &, const AdcStatus &) = default;

    QMap<QString, double> getBMTemps() const;

    std::optional<BMStatus> bmStatus;

    bool liveMagicCorrect = true;
    quint16 adcDesStatus = 0;
    bool adcSelfTestOk = true;
    int liveMagicResetCount = 0;
    AdcCoreStatus coreStatus;
    std::optional<ConfigKey> setup_key; // key of last configuration loaded to device

    AdcStatus & operator-=(const AdcStatus& st){
        if (wrStatus.has_value() && st.wrStatus.has_value())
            *wrStatus -= *st.wrStatus;
        liveMagicResetCount -= st.liveMagicResetCount;
        return *this;
    }
    void clear() {
        liveMagicCorrect = true;
        adcDesStatus = 0;
        adcSelfTestOk = true;
        liveMagicResetCount = 0;
    }
};
Q_DECLARE_METATYPE(AdcStatus)

#endif // ADCSTATUS_H
