/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef ADCCONFIGWIDGET_H
#define ADCCONFIGWIDGET_H

#include <QMap>
#include <QWidget>

#include "base/DeviceIndex.h"
#include "lib-common/BaseDeviceAppConfig.h"

namespace Ui {
class AdcConfigWidget;
}

struct AdcBuildParams;
class ConfigKey;
class FirFilterWidget;
class ModularDeviceStatus;
class RunLogicWidget;
class TrigCtrlStatWidget;
class WaveBlcWidget;

class AdcConfigWidget : public QWidget
{
    Q_OBJECT

public:
    explicit AdcConfigWidget(QWidget *parent = nullptr);
    ~AdcConfigWidget();

    void restoreFormValues();
    FirFilterWidget *getFirFilterWidget() const;
    RunLogicWidget *getRunLogicWidget() const;
    TrigCtrlStatWidget *getTrigCtrlStatWidget() const;
    WaveBlcWidget *getWaveBlcWidget() const;

signals:
    void configChanged(const BaseDeviceAppConfig &) const;
    void updateSparseMarkers();

public slots:
    void adcBuildParamsUpdated(DeviceIndex index, const AdcBuildParams &bp);
    void setConfig(const ConfigKey &key, const BaseDeviceAppConfig &_config);
    void deviceStatusUpdated(const ModularDeviceStatus &status);
    void setShowRunLogic(bool show);

protected:
    void changeEvent(QEvent *e);

protected slots:
    //--readout window
    void on_spinBoxSize_valueChanged(int newval);
    void on_spinBoxLatency_valueChanged(int newval);

private slots:
    void on_spinBoxPattern_valueChanged(int arg1);
    void on_checkBoxPattern_toggled(bool checked);

private:
    void setupChanged();
    bool updateDeviceBuildParams(DeviceIndex index, const AdcBuildParams &bp);
    void setTriggerSourceEnabled(TrigCtrlStatConfig::TrigSource trigSource, bool state);
    void updateLatMwSpinBox() const;
    void updateFirVisiblity() const;
    void updateRunLogicVisiblity() const;

    Ui::AdcConfigWidget *ui;
    QMap<DeviceIndex, AdcBuildParams> adcBuildParams;
    QMap<DeviceIndex, ModularDeviceStatus> deviceStatus;
    BaseDeviceAppConfig config;
    bool showRunLogic = false; // desirable state from Options menu
};

#endif // ADCCONFIGWIDGET_H
