#ifndef MONITOREVENTCOMBINER_H
#define MONITOREVENTCOMBINER_H

#include <QObject>

#include "base/DeviceIndex.h"

class QTimer;
class QElapsedTimer;

struct TqdcEvent;
struct CompleteEvent;
struct DecodedData;
class DeviceIndexEnabledMap;

class MonitorEventCombiner : public QObject
{
    Q_OBJECT
public:
    MonitorEventCombiner(QObject *parent = nullptr);
    ~MonitorEventCombiner() override;

signals:
    void monitorEventChanged(const CompleteEvent &);

public slots:
    void setDevices(const DeviceIndexEnabledMap &deviceList);
    void onMStreamDeviceDataUpdated(const DecodedData &data);
    void setMonitorDecodeEnabled(bool en);

private slots:
    void onDeviceDataAcquired(const DeviceIndex &index, const TqdcEvent &tqdcEvent);
    void eventTimeout();

private:
    void maybeSendMonitorEvent();

    bool enabled = true;
    DeviceIndexSet devSet;

    CompleteEvent * const eventData;
    CompleteEvent * const monitorEvent;
    QTimer * const eventTimeoutTimer;
    QElapsedTimer * const monitorTimeoutTimer;
    QElapsedTimer * const timestamp;
};

#endif // MONITOREVENTCOMBINER_H
