#include "FileReader.h"

#include <QFile>
#include <QFileInfo>
#include <QFileSystemWatcher>
#include <QDebug>

FileReader::FileReader(QObject *parent)
    : QObject(parent)
    , fileWatcher(new QFileSystemWatcher(this))
{
    connect(fileWatcher, &QFileSystemWatcher::fileChanged,
            this, &FileReader::checkFile);
}

void FileReader::setEnabled(bool en)
{
    if(enabled == en)
        return;

    enabled = en;
    lastFileData.clear();
    checkFile();
}

void FileReader::setConfig(const FileWatcherConfig &newConfig)
{
    if(cfg == newConfig)
        return;
    cfg = newConfig;

    fileWatcher->removePaths(fileWatcher->files());
    fileWatcher->addPath(cfg.filePath);
    qInfo().noquote() << QString("Looking for file: '%1'").arg(cfg.filePath);
    checkFile();
}

void FileReader::checkFile()
{
    if(cfg.filePath.isEmpty()) {
        emit fileValidUpdate(false);
        return;
    }

    QFile f(cfg.filePath);
    if(!f.open(QFile::ReadOnly)) {
        emit fileValidUpdate(false);
        return;
    }

    auto data = f.readAll();
    emit fileValidUpdate(!data.isNull());

    if(lastFileData == data)
        return;
    lastFileData = data;

    if(enabled)
        emit fileDataUpdate(data);
}
