#include "HrbDeviceGlobal.h"

#include "HrbDevice.h"

HrbDeviceGlobal::HrbDeviceGlobal(const DeviceIndex &index, QObject *parent)
    : MstreamMregDevGlobal(new HrbDevice(index), devStatus, parent)
{
}

void HrbDeviceGlobal::deviceStatusUpdated(const HrbStatus &status)
{
    Q_ASSERT(getDeviceIndex() == status.getDeviceIndex());
    if(getDeviceIndex() != status.getDeviceIndex())
        return;

    if(isRun) {
        HrbStatus diff = status;
        diff -= devStatus;
        isHrbStatusOk(diff);
    }
    devStatus = status;
}

bool HrbDeviceGlobal::isHrbStatusOk(const HrbStatus &status, bool silent)
{
    QString errWhat;
    if( !(status.rttcStatus & HrbStatus::RTTC_MASK_TTC_CLK))
        errWhat.append(QString("TTC_clk:fail; "));

    if(!(status.rttcStatus & HrbStatus::RTTC_MASK_TRIG_SYNC))
        errWhat.append(QString("TrigSync:fail; "));
    if(!(status.rttcStatus & HrbStatus::RTTC_MASK_RESET_SYNC))
        errWhat.append(QString("ResetSync:fail; "));
    //                diff.statistic.tsZeroMismatch ||
    if(status.statistic.ttcTrigSyncLoss)
        errWhat.append(QString("TrigSyncLosses=%1; ")
                       .arg(status.statistic.ttcTrigSyncLoss));
    if(status.statistic.ttcTrigCodeError)
        errWhat.append(QString("TrigCodeErrors=%1; ")
                       .arg(status.statistic.ttcTrigSyncLoss));
    if(status.statistic.ttcResetSyncLoss)
        errWhat.append(QString("ReseSyncLosses=%1; ")
                       .arg(status.statistic.ttcTrigSyncLoss));
    if(status.statistic.ttcResetCodeError)
        errWhat.append(QString("ResetCodeErrors=%1; ")
                       .arg(status.statistic.ttcTrigSyncLoss));

    bool ok = errWhat.isNull();
    if (!ok) {
        const QString &errStr = QString("[%1] Error in HRB statistic: %2")
                .arg(status.getDeviceIdent())
                .arg(errWhat);
        if (!silent) {
            // suppress repeated message
            if (errStr != errStrCache) {
                qWarning() << errStr;
            }
            emit runError(getDeviceIndex(), errStr);
        }
        errStrCache = errStr;
    } else {
        errStrCache = QString();
    }
    return ok;
}
