#include "AdjustBufferDialog.h"
#include "ui_AdjustBufferDialog.h"

#include "base/ClientIndexEnabled.h"

#include "AdjustBufferModel.h"
#include "SizeHelper.h"

#include <sys/sysinfo.h>
#include <QSignalBlocker>
#include <QTimer>

AdjustBufferDialog::AdjustBufferDialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::AdjustBufferDialog)
    , memTimer(new QTimer(this))
{
    ui->setupUi(this);
    connect(memTimer, &QTimer::timeout, this, &AdjustBufferDialog::upMemLabel);
    memTimer->setSingleShot(false);
    memTimer->start(1000);
}

AdjustBufferDialog::~AdjustBufferDialog()
{
    delete ui;
}

void AdjustBufferDialog::setModel(AdjustBufferModel *model) const
{
    ui->tableView->setModel(model);
}

void AdjustBufferDialog::setConfig(const EvBConfig &newConfig)
{
    if(config == newConfig)
        return;
    auto oldClientsMap = config.getClientIndexEnabledMap();
    auto newClientsMap = newConfig.getClientIndexEnabledMap();
    config = newConfig;

    QSignalBlocker b(ui->checkBoxAutoResize);
    ui->checkBoxAutoResize->setChecked(config.allowBufferAutoResize);
}

void AdjustBufferDialog::on_checkBoxAutoResize_toggled(bool checked)
{
    if(config.allowBufferAutoResize == checked)
        return;
    config.allowBufferAutoResize = checked;
    emit configChanged(config);
}

void AdjustBufferDialog::upMemLabel()
{
    QString txt;
    struct sysinfo info;
    int ret = sysinfo(&info);
    if(ret == 0) {
        txt = QString("Free: at least %1").arg(humanReadableSize(info.freeram * info.mem_unit));
    }
    ui->labelMem->setText(txt);
}
