#ifndef ADJUSTBUFFERMODEL_H
#define ADJUSTBUFFERMODEL_H

#include "mvc/ClientIndexTableModel.h"
#include <QObject>
#include <QMap>

class ClientIndex;
class AdjustBufferMap;
class ClientIndexEnabledMap;
struct ClientBufStatus;
struct EvBClientConfig;
struct EvbCombinerConfig;
struct EvBConfig;
struct EventSizePeriod;

class AdjustBufferModel : public ClientIndexTableModel
{
    Q_OBJECT
public:
    enum Columns{
        COL_TYPE, COL_INDEX,
        COL_CFG_SIZE,
        COL_PREF_SIZE, COL_CUR_SIZE,
        COL_TOTAL
    };
    AdjustBufferModel(QObject *parent = nullptr);
    ~AdjustBufferModel() override;

    virtual ClientIndex clientIndexOf(int row) const override;
    virtual int rowOf(const ClientIndex &sourceIndex) const override;

    virtual int rowCount(const QModelIndex &parent = QModelIndex()) const override;
    virtual int columnCount(const QModelIndex &parent = QModelIndex()) const override;

    virtual QVariant data(const QModelIndex &index, int role) const override;
    virtual bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
    virtual QVariant headerData(int section, Qt::Orientation orientation,
                                int role = Qt::DisplayRole) const override;
    Qt::ItemFlags flags(const QModelIndex &i) const override;

public slots:
    void setConfig(const EvBConfig &newConfig);
    void clientEvSizePeriodUpdated(const ClientIndex &i, const EventSizePeriod &period);
    void clientBuffSizeUpdated(const ClientIndex &i, quint64 size);
    void combinerEvSizePeriodUpdated(const EventSizePeriod &period);
    void combinerBuffSizeUpdated(int id, quint64 size);

signals:
    void configChanged(const EvBConfig &) const;

private:
    QVariant data_DisplayRole(const EvbCombinerConfig &cfg, const ClientBufStatus &st, int column) const;
    QVariant data_ToolTip(const EvbCombinerConfig &cfg, const ClientBufStatus &st, int column) const;

    QVariant data_EditRole(const EvBClientConfig &cfg, int column) const;
    QVariant data_DisplayRole(const EvBClientConfig &cfg, const ClientBufStatus &st, int column) const;
    QVariant data_ToolTip(const EvBClientConfig &cfg, const ClientBufStatus &st, int column) const;
    QVariant data_BackgroundRole(const EvBClientConfig &cfg, const ClientBufStatus &st, int column) const;
    QVariant data_BackgroundRole(quint64 bufSize, const ClientBufStatus &st, int column) const;
    void invalidateRow(const ClientIndex &i, const QVector<int> roles = {});
    void invalidateCombinerRow(const QVector<int> roles = {});

    bool autoResizeAll();
    bool autoResizeCombiner();
    bool autoResizeClient(const ClientIndex &i);

    AdjustBufferMap * const clients;
};

#endif // ADJUSTBUFFERMODEL_H
