#include "ClientFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.evb.ClientFsm")
}

ClientFsm::ClientFsm(QObject *parent)
    : BaseEvbClientFsm(parent)
{
    recreateInput(new ClientFsmInput());
}

void ClientFsm::setTcpState(FsmState tcpState)
{
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << tcpState;
    getInput()->tcpState = tcpState;
    evaluate();
}

void ClientFsm::setAnalizerState(FsmState state)
{
    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << state;
    getInput()->analizerState = state;
    evaluate();
}

void ClientFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

bool ClientFsm::checkSubStates(const FsmState &st) const
{
    auto i = getInput();
    if(i->analizerState!=st || i->tcpState!=st)
        return false;
    return true;
}

bool ClientFsm::hasAnyError() const
{
    auto i = getInput();
    if(i->analizerState == FsmState::Error || i->tcpState == FsmState::Error)
            return true;
    return false;
}

QDebug operator<<(QDebug dbg, const ClientFsmInput &in)
{
    dbg << BaseFsmInput(in);

    dbg << "tcp:" << in.tcpState << "; ";
    dbg << "analizer:" << in.analizerState << "; ";

    return dbg;
}
