#ifndef FILEWRITER_H
#define FILEWRITER_H

#include <QObject>
#include <QMetaType>
#include <QElapsedTimer>
#include <optional>

#include "base/RcRunId.h"
#include "base/ProgramIndex.h"

#include "EvbWriterConfig.h"
#include "FileWriterStats.h"
#include "RingBuffer.h"
#include "TcpServerStats.h"

class QFile;
class QTimer;
class ConfigIndexName;

Q_DECLARE_METATYPE(std::optional<int>)

class FileWriter : public QObject
{
    Q_OBJECT
public:
    explicit FileWriter(RingBuffer<char> * const writerRB, QObject *parent = nullptr);
    ~FileWriter() override;

signals:
    void statusUpdated(const FileWriterStats &) const;
    void fileOpened(bool opened, const RcRunId &runId) const;
    void error(int index, QString errStr); // index of file wrriter: -1
    void fileSplitted(uint id) const;

signals: void writerConfigChanged(const EvbWriterConfig &) const;
public slots: void setWriterConfig(const EvbWriterConfig &cfg);

signals: void switch_to_config(const ConfigIndexName &conf) const;
public slots: void configSelectorChanged(const ConfigIndexName &cin);

public slots:
    void clear();
    void startRun(const RcRunId &runId, std::optional<int> writerIndex, std::optional<int> splitIndex);
    void closeRun();
    void requestForSplit(uint id);

private slots:
    void writeData();
    void sendStatus();
    void sendMetrics();

private:
    bool openFileToWrite();
    void defineFileName();
    void closeFile();
    void dropRingBuffer();
    void updateStartTimeStamp();

    RcRunId runId;
    std::optional<int> writerIndex;
    std::optional<uint> splitIndex;
    EvbWriterConfig config;
    QFile * const dataFile;
    RingBuffer<char> * const ringBuf;
    FileWriterStats totalStats;
    TcpServerStats * const stats;
    QElapsedTimer writeLatencyElapsedTimer;
    QTimer * const writeTimer;
    QTimer * const statusTimer;
    QString startTimeStamp;
    ProgramIndex programIndex;
};

#endif // FILEWRITER_H
