#include "TcpClientFsm.h"

#include <QDebug>
#include <QLoggingCategory>

namespace {
Q_DECLARE_LOGGING_CATEGORY(fsm)
Q_LOGGING_CATEGORY(fsm, "afi.fsm.evb.tcpClient")
}

TcpClientFsm::TcpClientFsm(QObject *parent)
    : BaseEvbClientFsm(parent)
{
    recreateInput(new TcpClientFsmInput());
}

void TcpClientFsm::setRbCreated()
{
    if(getInput()->bufferCreated)
        return;

    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName();
    getInput()->bufferCreated = true;
    evaluate();
}

void TcpClientFsm::setConnected(bool connected)
{
    if(getInput()->connected == connected)
        return;

    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << connected;
    getInput()->connected = connected;
    evaluate();
}

void TcpClientFsm::setClientEn(bool clientEn)
{
    if(getInput()->clientEn == clientEn)
        return;

    qDebug(fsm) << __PRETTY_FUNCTION__ << objectName() << clientEn;
    getInput()->clientEn = clientEn;
    evaluate();
}

void TcpClientFsm::printInputString(QDebug dbg) const
{
    dbg << *getInput();
}

bool TcpClientFsm::hasAnyError() const
{
    auto i = getInput();
    return i->clientEn && (!i->connected || !i->bufferCreated);
}

QDebug operator<<(QDebug dbg, const TcpClientFsmInput &in)
{
    dbg << BaseFsmInput(in);

    dbg << "bufferCreated:" << in.bufferCreated << "; ";
    dbg << "connected:" << in.connected << "; ";
    dbg << "clientEn:" << in.clientEn << "; ";

    return dbg;
}
