#include "Tlu16SfpWidget.h"
#include "ui_Tlu16SfpWidget.h"
#include "Tlu16SfpModel.h"
#include "Tlu16SfpColumns.h"
#include "ItemDelegate/CommonSpinBoxDelegate.h"

#include "base/Globals.h"
namespace {
static const int THR_COL = static_cast<int>(Tlu16SfpColumns::COL_TOF_SUM_THR);
}

Tlu16SfpWidget::Tlu16SfpWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::Tlu16SfpWidget)
    , model(new Tlu16SfpModel(this))
    , thrSbDelegate(new CommonSpinBoxDelegate(this))
{
    ui->setupUi(this);
    ui->treeView->setModel(model);
    connect(model, &Tlu16SfpModel::modelReset, this, &Tlu16SfpWidget::onModelReset);

    ui->treeView->setItemDelegateForColumn(THR_COL, thrSbDelegate);

    ui->treeView->header()->setStretchLastSection(false);
    ui->treeView->header()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

Tlu16SfpWidget::~Tlu16SfpWidget()
{
    delete ui;
}

void Tlu16SfpWidget::updateRemoteBlocking(bool remoteConnected) const
{

}

void Tlu16SfpWidget::onFsmStateChange(const FsmState &state)
{
    bool en = state==FsmState::Ready || state==FsmState::Error
        || Globals::getDebugMode();
    setEnabled(en);
}

void Tlu16SfpWidget::onModelReset()
{
    addDelegate({});
    ui->treeView->expandAll();
    updateGeometry();

    for(auto c : {THR_COL}) {
        ui->treeView->resizeColumnToContents(c);
    }
}

void Tlu16SfpWidget::addDelegate(const QModelIndex &index)
{
    auto view = ui->treeView;
    for(auto c : {THR_COL}) {
        auto sibIndex = index.siblingAtColumn(c);
        if(!sibIndex.isValid())
            continue;
        if(!view->isPersistentEditorOpen(sibIndex))
            view->openPersistentEditor(sibIndex);
    }

    int chRows = model->rowCount(index);
    for(int i=0; i<chRows; ++i) {
        auto chIndex = model->index(i, 0, index);
        addDelegate(chIndex);
    }
}
