#ifndef MSCCORE_H
#define MSCCORE_H

#include <QObject>
#include <QMap>
#include <QSharedPointer>

#include "base/ConfigSelector.h"
#include "base/DeviceIndex.h"
#include "fsm/FsmState.h"
#include "lib-common/BaseDeviceAppConfig.h"
#include "lib-common/ReadoutAppCore.h"
#include "remote-control-server/RemoteControlServerStatus.h"

#include "status/MscStatus.h"

class ConfigKey;
class ConfigIndexName;
class ConfigSelector;
class MStreamDeviceMgrFsm;
class MscDeviceGlobal;
class MscStatusPublisher;
class ReadoutAppConfigDb;

class MscCore : public ReadoutAppCore
{
    Q_OBJECT
public:
    explicit MscCore(QObject *parent = nullptr);

signals:
    void configChanged(const BaseDeviceAppConfig &config);

public slots:
    void setConfig(const ConfigKey &config_key, const BaseDeviceAppConfig &newConfig);
    void statusUpdated(const MscStatus &status);
    void configSelectorChanged(const ConfigIndexName &conf) override;
    bool sendMetrics(const MscStatus &status);
    void onRemoteStatusUpdated(const RemoteControlServerStatus &status);
    void onFsmStateChange(const FsmState &state, const FsmState &oldState) override;

protected:
    const BaseFsm *getFsm() const override;

private:
    void addDevice(const DeviceIndex &index) override;
    void increaseMStreamSeqNum();
    void sendRunDocument() const;

    MStreamDeviceMgrFsm * const fsm;
    ReadoutAppConfigDb * const configDB;
    int mstreamSeqNum = 0;

    BaseDeviceAppConfig config;
    MscStatus status;
    ConfigSelector cs;
    DeviceIndex curDevice;
    QMap<DeviceIndex, QSharedPointer<MscDeviceGlobal> > devices;
    MscStatusPublisher * const statusPublisher;
    RemoteControlServerStatus lastRcStatus;
};

#endif // MSCCORE_H
