/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef FIRMWAREBUILDPARAMS_H
#define FIRMWAREBUILDPARAMS_H

#include <QMetaType>
#include <QtGlobal>

struct FirmwareBuildParams {
    bool isValid() const {
        return true
               && ((ver == 0) || (ver == 0x81))
               && ((nch > 0) && (nch <= 16))
               && ((ntrig > 0) && (ntrig <= 16))
               && ((spill_plot_ram_addr_bits > 0) && (spill_plot_ram_addr_bits <= 24))
               && ((trig_hist_ram_addr_bits > 0) && (trig_hist_ram_addr_bits <= 24))
               && ((level_cnt_bits >= 16) && (level_cnt_bits <= 64))
               && ((edge_cntr_bits >= 16) && (edge_cntr_bits <= 64))
               && ((clk_hz > 0) && (clk_hz <= 1e9))
            ;
    }
    quint8 ver = 0;
    bool edge_rollover = false;
    bool tih_log = false;
    bool multihist = false;
    quint8 nch = 1;
    quint8 ntrig = 1;
    quint32 spill_plot_ram_start_addr = 0;
    quint8 spill_plot_ram_addr_bits = 13;
    quint32 trig_hist_ram_start_addr= 0x10000;
    quint8 trig_hist_ram_addr_bits = 13;
    quint8 level_cnt_bits = 48;
    quint8 edge_cntr_bits = 32;
    quint32 clk_hz = 62.5e6;
};

Q_DECLARE_METATYPE(FirmwareBuildParams)

#endif // FIRMWAREBUILDPARAMS_H
