/*
**    Copyright 2021 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "StatusPublisher.h"

#include <QCoreApplication>
#include <QDebug>
#include <QJsonDocument>
#include <QTimer>

#include "RcSignalRouter.h"
#include "TrigStatus.h"
#include "TrigSpillStatistics.h"
#include "base/AbstractConfigConverter.h"
#include "base/DaqModule.h"
#include "base/DeviceIndex.h"
#include "message-bus/MessagePublisher.h"

constexpr const char k_state[] = "state";
constexpr const char k_fsm[] = "fsm";
constexpr const char k_dev[] = "dev";
constexpr const char k_stateStr[] = "stateStr";
constexpr const char k_status[] = "status";
constexpr const char k_spill_stats[] = "spill_stats";


StatusPublisher::StatusPublisher(QObject *parent)
    : QObject(parent)
    , appName(DaqModule(DaqModule::Type::RunControl).toString().toLower())
{
    AbstractConfigConverter<TrigStatus>::registerTypes();
    AbstractConfigConverter<TrigSpillStatistics>::registerTypes();

    auto &router = RcSignalRouter::instance();
    router.addStatusPublisher(this);
}

void StatusPublisher::pub(const QString &key, const QJsonDocument &json)
{
    MessagePublisher::instance().pub(key, json.toJson(QJsonDocument::Compact));
}

QStringList StatusPublisher::getDevStatePath(const DeviceIndex &index, QStringList suffix) const
{
    QString appName = DaqModule(DaqModule::Type::RunControl).toString().toLower();
    return QStringList{k_state, appName, k_dev, index.getDeviceIdName(),index.getSerialStr()}+suffix;
}
/*
void StatusPublisher::onRcFsmStateChange(const FsmState &state)
{
    pub(QStringList{k_state,appName,k_fsm,"total"}.join("/"), fsmStateToString(state));
}

void StatusPublisher::onClientManagerFsmStateChange(const FsmState &state)
{
    pub(QStringList{k_state,appName,k_fsm,"clients"}.join("/"), fsmStateToString(state));
}

void StatusPublisher::onTrigManagerFsmStateChange(const FsmState &state)
{
    pub(QStringList{k_state,appName,k_fsm,"trigMgr"}.join("/"), fsmStateToString(state));
}

void StatusPublisher::onVmeFsmStateChange(const FsmState &state)
{
    pub(QStringList{k_state,appName,k_fsm,"vme"}.join("/"), fsmStateToString(state));
}

void StatusPublisher::onTrigStatusStrUpdated(const DeviceIndex &index, const QString &st)
{
    pub(getDevStatePath(index, {k_stateStr}).join("/"), st);
}
*/
void StatusPublisher::onTrigStatusUpdated(const TrigStatus &status)
{
    const auto &json = QJsonDocument(QVariant::fromValue(status).toJsonObject());
    pub(getDevStatePath(status.getDeviceIndex(), {k_status}).join("/"), json);
}

//void StatusPublisher::onTrigFirmwareBuildParamsChanged(const FirmwareBuildParams &)
//{

//}

void StatusPublisher::onTrigSpillStatisticsUpdated(const DeviceIndex &index, const TrigSpillStatistics &data)
{
    QString key = getDevStatePath(index, {k_spill_stats}).join("/");
    const auto &json = QJsonDocument(QVariant::fromValue(data).toJsonObject());
    pub(key, json);
}
