//
//    Copyright 2014-2015 Ivan Filippov
//    Copyright 2014-2015 Ilja Slepnev
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef TRIGCHANNELCONFIG_H
#define TRIGCHANNELCONFIG_H

#include <QWidget>
#include <QMap>

#include "fsm/FsmState.h"

namespace Ui {
class TrigChannelConfig;
}
struct RcConfig;
class QTableWidget;
struct TrigChannelState;
class TrigChannelConfigPrivate;
class RcConfigDispatcher;

class TrigChannelConfig : public QObject
{
    Q_OBJECT

public:
    explicit TrigChannelConfig(QTableWidget *tableWidget = nullptr);
    ~TrigChannelConfig();
    void setConfigDispatcher(RcConfigDispatcher *p);

public slots:
    void updateChState(const TrigChannelState &chState);
    void setConfig(const RcConfig &newConfig);
    void setNumberOfChannels(int newChNum);
    void onRcFsmStateChange(FsmState st);

private slots:
    void onXOffToggled(bool checked);
    void tableCellChanged(int row, int col);

signals:
    void trigEnabled(int ch, bool enabled);
    void xOffEnabled(int ch, bool enabled);
    void configUpdated(quint16 xOffConf);
    void configChanged(const RcConfig &);

private:
    void updateGui();
    void setTableHeaders();
    void addRow(int row);
    int chToRow(int ch) const { return ch; }
    int rowToCh(int row) const { return row; }
    void updateTableSize();
    void updateColVisibility();
    bool xOffEnabled(int ch) const;

    Q_DECLARE_PRIVATE(TrigChannelConfig)
    TrigChannelConfigPrivate * const d_ptr;
    QTableWidget * const table;
    int chNum = 0;
    quint16 xOffConf = 0;
    quint16 xOffLed = 0;
    quint16 timeoutLed = 0;
};

#endif // TRIGCHANNELCONFIG_H
