/*
**    Copyright 2021 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "TrigSpillStatisticsConverter.h"

#include "TrigSpillStatistics.h"
#include "base/ConfigConverterUtil.h"

template<>
bool AbstractConfigConverter<TrigSpillStatistics>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TrigSpillStatistics>::toJsonObject(const TrigSpillStatistics &in)
{
    QVariantMap m;
    {
        QVariantMap spd;
        spd["data"] = ConfigConverterUtil::toJsonValue(in.spd.data);
        spd["spillCount"] = QVariant::fromValue(in.spd.spillCount);
        spd["spillCurrentTimeMs"] = QVariant::fromValue(in.spd.spillCurrentTimeMs);
        spd["stepSec"] = QVariant::fromValue(in.spd.stepSec);
        spd["startOffset"] = QVariant::fromValue(in.spd.startOffset);
        m["spd"] = spd;
    }
    {
        QVariantMap thd;
        QVariantMap histograms;
        for (auto it = in.thd.histograms.begin(); it != in.thd.histograms.end(); ++it) {
            QVariantMap m_cd;
            auto cd = it.value();
            m_cd["channel"] = QVariant::fromValue(cd.channel);
            m_cd["channelDescr"] = QVariant::fromValue(cd.channelDescr);
            m_cd["px"] = ConfigConverterUtil::toJsonValue(cd.px);
            m_cd["data"] = ConfigConverterUtil::toJsonValue(cd.data);
            histograms[QString::number(it.key())] = m_cd;
        }
        thd["histograms"] = histograms;
        thd["spillCount"] = QVariant::fromValue(in.thd.spillCount);
        thd["spillCurrentTimeMs"] = QVariant::fromValue(in.thd.spillCurrentTimeMs);
        m["thd"] = thd;
    }
    return fromVariantMap(m);
}

template<>
TrigSpillStatistics AbstractConfigConverter<TrigSpillStatistics>::fromJsonObject(const QJsonObject &)
{
    TrigSpillStatistics out;
    // not implemented
    return out;
}

template<>
QJsonValue AbstractConfigConverter<TrigSpillStatistics>::toJsonValue(const TrigSpillStatistics &in)
{
    return toJsonObject(in);
}

template<>
TrigSpillStatistics AbstractConfigConverter<TrigSpillStatistics>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TrigSpillStatistics>::registerSubTypes()
{
}
