/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef VMETRIGSTATUS_H
#define VMETRIGSTATUS_H

#include "ModularDeviceStatus.h"

struct VmeTrigCoreStatus
{
    friend bool operator==(const VmeTrigCoreStatus &, const VmeTrigCoreStatus &) = default;
    double occupancy() const { return (busyTimer!=0) ? (busyTimer/(busyTimer+idleTimer)) : 0; }
    quint16 deviceStatus = 0;
    quint64 evNum = 0;
    quint64 busyTimer = 0;
    quint64 idleTimer = 0;
};

struct VmeTrigStatus : public ModularDeviceStatus
{
    explicit VmeTrigStatus(const DeviceIndex &deviceIndex = {})
        : ModularDeviceStatus(deviceIndex) {}

    friend bool operator==(const VmeTrigStatus &, const VmeTrigStatus &) = default;
    bool isWrTimeValid() const { return coreStatus.deviceStatus & 0x1; }
    void clear() {
        wrStatus = {};
        coreStatus = {};
    }
    QString ident;
    VmeTrigCoreStatus coreStatus;
};

Q_DECLARE_METATYPE(VmeTrigStatus)

#endif // VMETRIGSTATUS_H
