/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef TQDCDACCALIBRATOR_H
#define TQDCDACCALIBRATOR_H

#include "device-utils/AbstractThresholdCalibrator.h"

#include "cal/CalResult.h"
#include <optional>
#include <QSharedPointer>

class HitCoreModule;
class Ad5328ConfigBuilder;
class Ad5328Module;

class TqdcDacCalibrator : public AbstractThresholdCalibrator
{
public:
    explicit TqdcDacCalibrator(const Ad5328ConfigBuilder &dacConfigBuilder)
        : dacConfigBuilder(dacConfigBuilder) {}

    void setModules(QSharedPointer<HitCoreModule> hitCore,
                    QSharedPointer<Ad5328Module> thresholdModule)
    {
        this->hitCore = hitCore;
        this->thresholdMod = thresholdModule;
    }

    std::optional<DacCalResult> calibrateDac();

protected:
    bool writeLevels(const QMap<int, int> &values) override;
    bool readStates(AllChannelState *states) override;
    double convert_raw_to_mV(double raw) const override;
    int getNumberOfChannels() const override;

private:
    QSharedPointer<HitCoreModule> hitCore;
    QSharedPointer<Ad5328Module> thresholdMod;
    const Ad5328ConfigBuilder &dacConfigBuilder;
    const int num_samples = 20;
};

#endif // TQDCDACCALIBRATOR_H
