#include "TqdcTelnetRemoteControl.h"

#include <QDebug>

TqdcTelnetRemoteControl::TqdcTelnetRemoteControl(quint16 port, QObject *parent)
    : TelnetRemoteControl(port)
{
}

void TqdcTelnetRemoteControl::setConfig(const ConfigKey &configKey, const BaseDeviceAppConfig &newConfig)
{
    Q_UNUSED(configKey);
    if (config == newConfig)
        return;
    config = newConfig;
}

void TqdcTelnetRemoteControl::setParam(const QString &key, const QVariant &val)
{
    static const constexpr char* PAR_LAT = "lat";
    static const constexpr char* PAR_MW = "matchWin";
    static const constexpr char* PAR_RUN_TIME = "runTimeLimit";

    if(key == PAR_LAT || key == PAR_MW) {
        bool ok;
        int valInt = val.toInt(&ok);
        if(!ok) {
            qWarning() << QString("Invalid value (%1) for key (%2)")
                          .arg(val.toString()).arg(key);
            return;
        }
        auto &devCfg = config.defaultSetup.tqdcCore;
        auto &devCfgVal = key == PAR_LAT ? devCfg.latency :devCfg.matchWin;
//        auto sb = key == PAR_LAT ? ui->spinBoxLatency : ui->spinBoxMatchWin;

//        if(valInt < sb->minimum() || valInt > sb->maximum()) {
//            qWarning() << QString("Can't set key(%1) valu(%2) out of range %3-%4")
//                          .arg(key).arg(valInt).arg(sb->minimum()).arg(sb->maximum());
//            return;
//        }

        devCfgVal = valInt;
        config.unify_config();
        emit configChanged(config);
    } else if(key == PAR_RUN_TIME) {
        bool ok;
        uint valUInt = val.toUInt(&ok);
        if(!ok) {
            qWarning() << QString("Invalid value (%1) for key (%2)")
                          .arg(val.toString()).arg(key);
            return;
        }
        auto &rl = config.defaultSetup.runLogic;
        if(valUInt)
            rl.timeLimit = valUInt;
        rl.timeLimitEnable = valUInt;
        config.unify_config();
        emit configChanged(config);
    } else {
        sendWarn(QString("Undefined remote param(%1) use one of these: ")
                 .arg(key).arg(QStringList{PAR_LAT, PAR_MW}.join(' ')));
    }
}
