﻿#include "DeviceChTableView.h"

#include <QHeaderView>

#include "TrcTreeColumns.h"
#include "TrcSpinBoxDelegate.h"

namespace {
const static QVector<TrcTreeColumns> DELEGATE_COLS = {
    TrcTreeColumns::COL_DELAY,
    TrcTreeColumns::COL_REDUC_FACTOR,
    TrcTreeColumns::COL_AFTER_TIME,
    TrcTreeColumns::COL_BEFORE_TIME,
};
}

DeviceChTableView::DeviceChTableView(QWidget *parent)
    : QTableView(parent)
    , delegate(new TrcSpinBoxDelegate(this))
{
    horizontalHeader()->setStretchLastSection(false);
    horizontalHeader()->setSectionResizeMode(QHeaderView::ResizeToContents);
}

void DeviceChTableView::setModel(QAbstractItemModel *model)
{
    QTableView::setModel(model);
    for(auto c : DELEGATE_COLS)
        setItemDelegateForColumn(static_cast<int>(c), delegate);

    connect(model, &QAbstractItemModel::modelReset, this, &DeviceChTableView::onModelReset);
    addDelegates();
}

void DeviceChTableView::onModelReset()
{
    addDelegates();
}

void DeviceChTableView::addDelegates()
{
    const auto &m = model();
    int rows = m->rowCount();
    for(int r=0; r<rows; ++r) {
        for(auto c : DELEGATE_COLS) {
            auto i = m->index(r, static_cast<int>(c));
            if(!isPersistentEditorOpen(i))
                openPersistentEditor(i);
        }
    }
}
