#include "AfiViewerConfigDispatcher.h"

#include "base/util.h"

AfiViewerConfigDispatcher::AfiViewerConfigDispatcher(QObject *parent) :
    BaseConfigDispatcher(parent),
    cache(new AfiViewerConfig())
{
}

void AfiViewerConfigDispatcher::setConfig(const AfiViewerConfig &newConfig)
{
    if (newConfig == *cache)
        return;
    bool histChanged = !(newConfig.hist_manager_config == cache->hist_manager_config);
    bool zmqChanged = !(newConfig.zmq_config == cache->zmq_config);
    *cache = newConfig;
    qDebug() << "configChanged by" << qObjectAncestryStr(sender());
    emit configChanged(*cache);
    Q_ASSERT(!newConfig.hist_manager_config.hist_groups.contains(HistId::unknown));
    if(histChanged)
        emit histConfigChanged(cache->hist_manager_config);
    if(zmqChanged)
        emit zmqConfigChanged(cache->zmq_config);
    emit fileReaderConfigChanged(cache->file_reader_config);
    emit waveformViewerConfigChanged(cache->wf_config);
}

void AfiViewerConfigDispatcher::setFileReaderConfig(const FileReaderConfig &newConfig)
{
    if(cache->file_reader_config == newConfig)
        return;

    cache->file_reader_config = newConfig;
    if(!newConfig.lastFile.isEmpty())
        cache->source = AfiViewerConfig::SOURCE_DATA::SOURCE_FILE;

    emit fileReaderConfigChanged(cache->file_reader_config);
    emit configChanged(*cache);
}

void AfiViewerConfigDispatcher::setHistConfig(const HistMngrCfg &newConfig)
{
    if (newConfig == cache->hist_manager_config)
        return;
    cache->hist_manager_config = newConfig;
    qDebug() << "histConfigChanged by" << qObjectAncestryStr(sender());
    emit histConfigChanged(cache->hist_manager_config);
    emit configChanged(*cache);
}

void AfiViewerConfigDispatcher::setWaveformViewerConfig(const WaveformViewerConfig &newConfig)
{
    if (cache->wf_config == newConfig)
        return;
    cache->wf_config = newConfig;
    qDebug() << "waveformViewerChanged by" << qObjectAncestryStr(sender());
    emit waveformViewerConfigChanged(cache->wf_config);
    emit configChanged(*cache);
}

void AfiViewerConfigDispatcher::setZmqConfig(const ZmqConfig &newConfig)
{
    if (newConfig == cache->zmq_config)
        return;
    cache->zmq_config = newConfig;
    qDebug() << "zmqConfigChanged by" << qObjectAncestryStr(sender());
    emit zmqConfigChanged(cache->zmq_config);
    emit configChanged(*cache);
}
