#ifndef WAVEFORMVIEWER_H
#define WAVEFORMVIEWER_H

#include <QWidget>
#include <optional>

#include "histogramm/DeviceData.h"

#include "WaveformViewerConfig.h"

namespace Ui {
class WaveformViewer;
}

class QwtPlotMarker;

class WaveformViewer : public QWidget
{
    Q_OBJECT

public:
    explicit WaveformViewer(QWidget *parent = nullptr);
    ~WaveformViewer() override;

signals:
    void waveformViewerConfigChanged(const WaveformViewerConfig &cfg) const;

public slots:
    void deviceDataUpdated(const DecodedData &data,
                           bool writeDecodedEnabled);
    void refreshPlot(bool force = false);
    void setWaveformViewerConfig(const WaveformViewerConfig &config = WaveformViewerConfig());
    void reset();

private slots:
    void on_checkBoxEn_toggled(bool checked);
    void on_comboBoxDev_currentIndexChanged(int index);
    void on_spinBoxChMin_valueChanged(int arg1);
    void on_spinBoxChMax_valueChanged(int arg1);
    void on_spinBoxSparseMarks_valueChanged(int p);

private:
    void initGui();
    void addDevice(const DeviceIndex &index);
    void showWaveform(const CompleteEvent &ev);
    bool hasSparseMarkers() const;
    void updateMarkers();
    int getMaxCh(const DeviceIndex &i={}) const;
    void updateMaxMinLimits();

    Ui::WaveformViewer *ui;
    DecodedData lastData;
    DecodedData pendingData;
    bool hasPendingData;
    QTimer * const refreshTimer;
    WaveformViewerConfig cfg;
    DeviceIndexSet addedDevices;
    QHash<DeviceIndex, int> devMaxCh;
    bool hasSeveralDev = false;
    bool hasSeveralTypes = false;
    QVector<QwtPlotMarker *> sparseMarkers;
    std::optional<QPair<double,double>> wfRange;
};

#endif // WAVEFORMVIEWER_H
