/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ClientIndex.h"

#include <QDataStream>
#include <QDebug>

bool ClientIndex::operator ==(const ClientIndex &index) const
{
    return this->program_type == index.program_type &&
        this->program_index == index.program_index;
}

bool ClientIndex::operator <(const ClientIndex &r) const
{
    if (program_type == r.program_type)
        return (program_index < r.program_index);
    return (program_type < r.program_type);
}

QString ClientIndex::toString() const
{
    return QString("%1#%2").arg(program_type.toString()).arg(program_index.toString());
}

ClientIndex ClientIndex::fromString(const QString &r)
{
    const auto &subStr = r.split("#");
    return subStr.size() < 2 ? ClientIndex() :
                               ClientIndex(DaqModule::fromString(subStr.at(0)),
                                         ProgramIndex::fromString(subStr.at(1)));
}

QDebug operator<<(QDebug dbg, const ClientIndex &r)
{
    dbg.noquote().nospace() << QString("ClientIndex(%1, %2)")
                                   .arg(r.program_type.toString())
                                   .arg(r.program_index.toString());
    return dbg;
}

QDataStream &operator<<(QDataStream &out, const ClientIndex &value)
{
    out << value.program_type  << value.program_index;
    return out;
}

QDataStream &operator>>(QDataStream &in, ClientIndex &value)
{
    DaqModule daqModule;
    ProgramIndex programIndex;
    in >> daqModule >> programIndex;
    if (in.status() == QDataStream::Ok) {
        value.program_type = daqModule;
        value.program_index = programIndex;
    }
    return in;
}

ClientIndexSet makeClientIndexSet(const QList<ClientIndex> &list)
{
    return ClientIndexSet(list.constBegin(), list.constEnd());
}
