/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ClientIndexConverter.h"

#include "DaqModuleConverter.h"
#include "ProgramIndexConverter.h"


namespace {
constexpr const char *k_program_type = "program_type";
constexpr const char *k_program_index = "program_index";
}

template<>
bool AbstractConfigConverter<ClientIndex>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<ClientIndex>::toJsonObject(const ClientIndex &in)
{
    QVariantMap m;
    m[k_program_type] = QVariant::fromValue(in.program_type);
    m[k_program_index] = QVariant::fromValue(in.program_index);
    return fromVariantMap(m);
}

template<>
ClientIndex AbstractConfigConverter<ClientIndex>::fromJsonObject(const QJsonObject &m)
{
    ClientIndex out;
    out.program_type = QVariant::fromValue(m[k_program_type]).value<DaqModule>();
    out.program_index = QVariant::fromValue(m[k_program_index]).value<ProgramIndex>();
    return out;
}

template<>
QJsonValue AbstractConfigConverter<ClientIndex>::toJsonValue(const ClientIndex &in)
{
    return toJsonObject(in);
}

template<>
ClientIndex AbstractConfigConverter<ClientIndex>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<ClientIndex>::registerSubTypes()
{
    AbstractConfigConverter<DaqModule>::registerTypes();
    AbstractConfigConverter<ProgramIndex>::registerTypes();
#if (QT_VERSION < QT_VERSION_CHECK(6,0,0))
    if (QMetaType::hasRegisteredComparators<ClientIndex>())
        return;
    qRegisterMetaTypeStreamOperators<ClientIndex>();
    QMetaType::registerComparators<ClientIndex>();
#endif

    QMetaType::registerConverter<ClientIndex, QString>(&ClientIndex::toString);
    QMetaType::registerConverter<QString, ClientIndex>(&ClientIndex::fromString);
}
