/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "fw_version.h"

#include <QString>
#include <QStringList>

QString fw_version_t::toString() const
{
    return toString(*this);
}

QString fw_version_t::toString(const fw_version_t &r)
{
    QString s = QString("%1.%2").arg(r.v_maj).arg(r.v_min);
    if (r.revision != 0)
        s += QString(".%1").arg(r.revision);
    return s;
}

fw_version_t fw_version_t::fromString(const QString &r)
{
    const auto &list = r.toLower().split(".");
    if (list.size() < 1)
        return {};
    bool ok;

    int v_maj = 0;
    v_maj = list[0].toUInt(&ok, 0);
    Q_ASSERT(ok);
    if (!ok)
        return {};

    int v_min = 0;
    if (list.size() >= 1) {
        v_min = list[1].toUInt(&ok, 0);
        Q_ASSERT(ok);
        if (!ok)
            return {};
    }

    int revision = 0;
    if (list.size() >= 2) {
        revision = list[2].toUInt(&ok, 0);
        Q_ASSERT(ok);
        if (!ok)
            return {};
    }

    return fw_version_t(v_maj, v_min, revision);
}
