/*
**    Copyright 2009-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DSPLIB_FITTERGAUS_H
#define DSPLIB_FITTERGAUS_H

#include "FitterBase.h"
#include <vector>

namespace dsplib {

class FitterGaus : public FitterBase
{
public:
    explicit FitterGaus (unsigned int fit_points);
    int fit (int fixpar, const double *x, int len, double *par, fitstatus_t *fs); // not reentrant
};

// Coarse approximation of gauss curve (return chi2)
void FindGausApprox (const int *x, int len, double *par);
double FindGausDevNorm (const int *x, int len, double *par);
void FindGausApproxD (const double *x, int len, double *par);
double FindGausDevNormD (const double *x, int len, double *par);

int QuickFitGaus (const double *x, int len, double *par, fitstatus_t *fs);

// New interface

class GausPar
{
public:
    GausPar(double p0 = 0, double p1 = 0, double p2 = 0) : a(p0), x0(p1), sigma(p2) { }
    explicit GausPar(const double p[3]) { set(p); }
    void get(double p[3]) const { p[0] = a; p[1] = x0; p[2] = sigma; }
    void set(const double p[3]) { a = p[0]; x0 = p[1]; sigma = p[2]; }
    bool operator==(const GausPar &r) const { return a==r.a && x0==r.x0 && sigma== r.sigma; }
    double a = 0;
    double x0 = 0;
    double sigma = 0;
};

double GausFcn(const GausPar &par, double x);

class GaussianApproximator
{
public:
    // Coarse approximation of gauss curve (return chi2)
    static GausPar FindGausApprox (const std::vector<int> &x);
    static GausPar FindGausApprox (const std::vector<double> &x);
    static GausPar FindGausApproxShifted (const std::vector<int> &x);
    static GausPar FindGausApproxShifted (const std::vector<double> &x);
    static double FindGausDevNorm (const std::vector<int> &x, const GausPar &par);
    static double FindGausDevNorm (const std::vector<double> &x, const GausPar &par);
    static double FindGausDevNormShifted (const std::vector<int> &x, const GausPar &par);
    static double FindGausDevNormShifted (const std::vector<double> &x, const GausPar &par);
};

} // namespace dsplib

#endif // DSPLIB_FITTERGAUS_H
