/*
**    Copyright 2010-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef DSPLIB_RFFTENGINE_H
#define DSPLIB_RFFTENGINE_H

#include <map>
#include <vector>
#include <complex> /* complex must be included before fftw3.h */
#include <fftw3.h>

namespace dsplib {

class RFFTEngine {
public:
    virtual ~RFFTEngine();
    std::vector<std::complex<double> > DFT(const std::vector<double> &d) const; // real to complex
    std::vector<double> IDFT(const std::vector<std::complex<double> > &d) const; // complex to real
    //        static std::vector<FFTSpectrumPeak> getSpectrumPeaks(const FFTSpectrum &, double threshold = 0.1);

protected:
    size_t getCacheSizeLimit() const;
    fftw_plan get_r2c(size_t N) const;
    fftw_plan get_c2r(size_t N) const;

    mutable std::map<size_t, fftw_plan> r2cCache, c2rCache;
};

} // namespace dsplib

#endif /* DSPLIB_RFFTENGINE_H */
