/*
**    Copyright 2021 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef JSONRPCTYPES_H
#define JSONRPCTYPES_H

#include <QJsonValue>

typedef long RpcRequestId;

enum class JsonRpcError {
    Timeout = -62,
    ParseError = -32700,
    InvalidRequest = -32600,
    MethodNotFound = -32601,
    InvalidParams = -32602,
    InternalError = -32603,
    ApplicationError = -32500,
    SystemError = -32400,
    TransportError = -32300
};

struct JsonRpcRequest
{
    JsonRpcRequest(const QString &method = {}, const QJsonValue &params = {})
        : method(method), params(params) {}
    QString toString(bool compact = true) const;
    QString method;
    QJsonValue params;
};

Q_DECLARE_METATYPE(JsonRpcRequest)

#endif // JSONRPCTYPES_H
