/*
**    Copyright 2015-2020 Ivan Filippov
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "ManualAddDialog.h"
#include "ui_ManualAddDialog.h"
#include <QRegularExpressionValidator>
#include <QDebug>
#include <QDialogButtonBox>
#include <QPushButton>

ManualAddDialog::ManualAddDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ManualAddDialog)
{
    ui->setupUi(this);
    QRegularExpression rx("[0-9a-fA-F]{1,12}");
    auto *rxVal = new QRegularExpressionValidator(rx, this);
    ui->lineEditSerial->setValidator(rxVal);

    QRegularExpression rx2("[0-9a-fA-F]{1,2}");
    auto *rxVal2 = new QRegularExpressionValidator(rx2, this);
    ui->comboBoxDeviceId->setValidator(rxVal2);

    connect(ui->lineEditSerial, &QLineEdit::textEdited,
            this, &ManualAddDialog::validateData);
    connect(ui->comboBoxDeviceId, &QComboBox::currentTextChanged,
            this, &ManualAddDialog::validateData);
    validateData();
}

quint64 ManualAddDialog::getSerial() const
{
    const auto text = ui->lineEditSerial->text();
    bool ok;
    quint64 serial = text.toULongLong(&ok);
    if(ok) return  serial;
    serial = text.toULongLong(&ok, 16);
    return ok ? serial : 0;
}

quint16 ManualAddDialog::getDeviceId() const
{
    const QString txt = ui->comboBoxDeviceId->currentText();
    for (quint16 id : deviceTypeMap.keys()) {
        if(deviceTypeMap[id] == txt)
            return id;
    }

    bool ok;
    quint16 deviceId = static_cast<quint16>(txt.toUInt(&ok));
    if(ok) return deviceId;
    deviceId = static_cast<quint16>(txt.toUInt(&ok, 16));
    return ok ? deviceId : 0;
}

void ManualAddDialog::setDeviceIdMap(QMap<quint16, QString> deviceTypeMap)
{
    this->deviceTypeMap = deviceTypeMap;
    ui->comboBoxDeviceId->clear();
    for (const quint16 id : deviceTypeMap.keys()) {
        ui->comboBoxDeviceId->insertItem(0, deviceTypeMap[id], id);
    }
}

void ManualAddDialog::validateData() const
{
    ui->buttonBox->button(QDialogButtonBox::Ok)->setEnabled(getDeviceId() && getSerial());
}

ManualAddDialog::~ManualAddDialog()
{
    delete ui;
}
