/*
**    LLDP Protocol, M-Link OUI specific TLVs
**
**    See IEEE 802.1AB (Station and Media Access Control Connectivity Discovery)
**    See ANSI/TIA-1057 LLDP-MED (Media Endpoint Discovery)
**
**    Copyright 2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef MLDP_H
#define MLDP_H

#include <stdint.h>

#pragma pack(push, 1)

struct afi_tlv_1_t {
    uint16_t device_id : 16;
    uint64_t serial_id : 48;
} __attribute__((packed));

struct afi_tlv_1_ext_t {
    uint16_t device_id : 16;
    uint64_t serial_id : 48;
    uint64_t chassis_slot : 16;
} __attribute__((packed));

struct afi_tlv_2_t {
    uint64_t master_mac : 48;
    uint32_t master_ip : 32;
    uint16_t master_udp_port : 16;
    uint64_t mstream_mac : 48;
    uint32_t mstream_ip : 32;
    uint16_t mstream_udp_port : 16;
    struct {
        uint8_t master_locked : 1;
        uint8_t mstream_locked : 1;
        uint8_t unused: 6;
    } flags;
} __attribute__((packed));

#pragma pack(pop)

#endif // MLDP_H
