#include "Ddr3StreamFifoModule.h"

#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

#include "Ddr3StreamFifoStatus.h"

// MCU SPI Slave
enum {
    REG_CTRL             = 0x00, // R/W
    REG_STATUS_CODE      = 0x01,
    REG_FULL_SIZE        = 0x02,
    REG64_FIFO_OCCUP     = 0x04, // 64-bit
    REG32_ECC_MULTI_ERR  = 0x08, // 32-bit
    REG32_ECC_SINGLE_ERR = 0x0A, // 32-bit
};

QList<AFI_SDB_ID> Ddr3StreamFifoModule::getSupportedSdbIds()
{
    return {AFI_SDB_DDR3_Stream_FIFO};
}

mlink::RegOpVector Ddr3StreamFifoModule::getStatusRegOp(Ddr3StreamFifoStatus *result) const
{
    RegOpVector r;
    r.RegWrite16(REG_CTRL, 0x2);
    r.RegRead16(REG_STATUS_CODE, &result->statusCode.all);
    r.RegRead16(REG_FULL_SIZE, &result->fullMemRaw);
    r.RegRead64(REG64_FIFO_OCCUP, &result->fifoOccup);
    r.RegRead32(REG32_ECC_MULTI_ERR, &result->eccMulti);
    r.RegRead32(REG32_ECC_SINGLE_ERR, &result->eccSingle);
    r.RegWrite16(REG_CTRL, 0x0);
    return r;
}

std::optional<Ddr3StreamFifoStatus> Ddr3StreamFifoModule::readStatus()
{
    Ddr3StreamFifoStatus result;
    if (!readStatus(&result))
        return {};
    return result;
}

bool Ddr3StreamFifoModule::readStatus(Ddr3StreamFifoStatus *result)
{
    auto r = getStatusRegOp(result);
    return regOpExecRebased(r);
}
