#include "FirFilterModule.h"

#include "regio/regop.h"

#include "FirFilterConfig.h"
#include "sdb/AfiSdbId.h"

enum {
    REG32_ID        = 0x0,
    REG_CONTROL     = 0x2,
    REG_COEF_CTRL   = 0x3,
    REG_ROUNDOFF    = 0x4,
    REG_TLAST_MISSING    = 0xA,
    REG_TLAST_UNEXPECTED = 0xB,
    REG64_BUILDPARAMS = 0xC,
    REG_COEF_START  = 0x20,
};

static const quint16 REG_CONTROL_BIT_RESET = 0x1;
static const quint16 REG_CONTROL_BIT_ENABLE = 0x8000;

QList<AFI_SDB_ID> FirFilterModule::getSupportedSdbIds()
{
    return { AFI_SDB_FIR_FILTER };
}

void FirFilterModule::post_init()
{
    buildParams.reset();
    FirFilterBuildParams bp;
    quint64 buildparams = 0;
    RegOpVector r;
    r.RegRead32(REG32_ID, &bp.id);
    r.RegRead64(REG64_BUILDPARAMS, &buildparams);
    if (regOpExecRebased(r)) {
        bp.size = buildparams & 0xFF;
        bp.int_roundoff = (buildparams >> 8) & 0xFF;
        buildParams = bp;
    }
}

bool FirFilterModule::writeConfig(const FirFilterConfig &fir)
{
    if (!buildParams || !buildParams->isValid())
        return {};

    RegOpVector r;
    r.RegWrite16(REG_ROUNDOFF, fir.roundoff);
    int num_coef = std::min(fir.coef.size(), (int)buildParams->size);
    for (int i=0; i<num_coef; i++) {
        r.RegWrite16(REG_COEF_START+i, fir.coef[i]);
    }
    for (int i=num_coef; i<buildParams->size; i++) {
        r.RegWrite16(REG_COEF_START+i, 0);
    }
    r.RegWrite16(REG_COEF_CTRL, REG_CONTROL_BIT_RESET);
    r.RegWrite16(REG_COEF_CTRL, 0);
    r.RegWrite16(REG_CONTROL, fir.isEnabled() ? REG_CONTROL_BIT_ENABLE : 0);
    return regOpExecRebased(r);
}

std::optional<FirFilterStatus> FirFilterModule::readStatus()
{
    FirFilterStatus status;
    if (!readStatus(&status))
        return {};
    return status;
}

bool FirFilterModule::readStatus(FirFilterStatus *result)
{
    if (!buildParams || !buildParams->isValid()) {
        return true;
        return {};
    }

    FirFilterStatus status;

    RegOpVector r;
    r.RegRead16(REG_TLAST_MISSING, &status.missing);
    r.RegRead16(REG_TLAST_UNEXPECTED, &status.unexpected);

    if (!regOpExecRebased(r))
        return {};

    if (result)
        *result = status;
    return true;
}
