/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "FlashProgModuleConfigConverter.h"
#include "base/AbstractConfigConverter.h"
#include "device-modules/FlashProg/FlashProgModuleConfig.h"

namespace {
constexpr const char *k_filePath = "filePath";
} // namespace

//  ==== FlashProgModuleConfig start ====
template<>
bool AbstractConfigConverter<FlashProgModuleConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<FlashProgModuleConfig>::toJsonObject(const FlashProgModuleConfig &in)
{
    QVariantMap m;
    m[k_filePath] = QVariant::fromValue(in.imageFilePath);
    return fromVariantMap(m);
}

template<>
FlashProgModuleConfig AbstractConfigConverter<FlashProgModuleConfig>::fromJsonObject(const QJsonObject &in)
{
    FlashProgModuleConfig out;
    update_value(in, k_filePath, &out.imageFilePath);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<FlashProgModuleConfig>::toJsonValue(const FlashProgModuleConfig &in)
{
    return toJsonObject(in);
}

template<>
FlashProgModuleConfig AbstractConfigConverter<FlashProgModuleConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<FlashProgModuleConfig>::registerSubTypes()
{}

//  ==== FlashProgModuleConfig end   ====
