#ifndef MSCCYCLESTATUS_H
#define MSCCYCLESTATUS_H

#include <QVector>
#include <QMetaType>

#include <optional>

#include "base/TaiTime.h"

struct MscGateStatus
{
    friend bool operator==(const MscGateStatus &, const MscGateStatus &) = default;

    TaiTime lastGateStart() const;
    TaiTime lastGateEnd() const;

    quint32 activeGateCnt = 0; // Reg 0x10 - counts during active gate, 32-bits
    quint32 fullGateCnt   = 0; // Reg 0x12 - counts for full cycle (active + inactive gate), 32-bits
    quint64 lastGateStartRaw;      // Reg 0x14 - last gate start time, TAI 64-bit Timestamp, 64-bits
    quint64 lastGateEndRaw;        // Reg 0x18 - last gate end time, TAI 64-bit Timestamp, 64-bits
    quint32 gateCnt       = 0; // Reg 0x1C - gate number, 32-bits
    std::optional<qint64> lastGateWidth;  // start-prevStart in ns (s/w calculated)
    std::optional<qint64> lastGatePeriod; // end-start       in ns (s/w calculated)
};

struct MscCycleStatus
{
    friend bool operator==(const MscCycleStatus &, const MscCycleStatus &) = default;

    bool isInGate(int hId) const {
        return curGateState & (1<<hId);
    }
    quint64 curGateState = 0; // Reg 0x4 - Current state of cycles, 64-bits, RO
    QVector<MscGateStatus> gates; // by hId
};
Q_DECLARE_METATYPE(MscCycleStatus)

#endif // MSCCYCLESTATUS_H
