/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "NetworkPortConfigConverter.h"

#include "base/ConfigConverterUtil.h"

template<>
bool AbstractConfigConverter<NetworkPortConfig>::classIsObject()
{
    return false;
}

template<>
QJsonObject AbstractConfigConverter<NetworkPortConfig>::toJsonObject(const NetworkPortConfig &in)
{
    QVariantMap m;

    m["mstreamReset"] = QVariant::fromValue(in.mstreamReset);
    m["mtu"] = QVariant::fromValue(in.mtu);
    m["defragTimeoutUs"] = QVariant::fromValue(in.defragTimeoutUs);
    m["retxTimeoutUs"] = QVariant::fromValue(in.retxTimeoutUs);

    return fromVariantMap(m);
}

template<>
NetworkPortConfig AbstractConfigConverter<NetworkPortConfig>::fromJsonObject(const QJsonObject &in)
{
    NetworkPortConfig out;

    update_value(in, "mstreamReset", &out.mstreamReset);
    update_value(in, "mtu", &out.mtu);
    update_value(in, "defragTimeoutUs", &out.defragTimeoutUs);
    update_value(in, "retxTimeoutUs", &out.retxTimeoutUs);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<NetworkPortConfig>::toJsonValue(const NetworkPortConfig &in)
{
    return toJsonObject(in);
}

template<>
NetworkPortConfig AbstractConfigConverter<NetworkPortConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<NetworkPortConfig>::registerSubTypes()
{
}
