/*
**    Copyright 2014-2018 Ivan Filippov
**    Copyright 2014-2020 Ilja Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "RcTrigConfig.h"

#include <QDataStream>
#include <QDebug>

bool RcTrigConfig::operator==(const RcTrigConfig &rhs) const
{
    return true
            && xOffMask == rhs.xOffMask
            && trigOpen == rhs.trigOpen
            && trigSrc == rhs.trigSrc
            && qFuzzyCompare(trigTimerFreq, rhs.trigTimerFreq)
            && qFuzzyCompare(trigRandomFreq, rhs.trigRandomFreq)
            && qFuzzyCompare(trigRandomDeadTimeUs, rhs.trigRandomDeadTimeUs)
            ;
}

QDebug operator <<(QDebug stream, const RcTrigConfig &trig)
{
    stream.noquote().nospace()
            << "TrigConfig("
            << QString("XOFF mask: %1, ").arg(trig.xOffMask, 8, 16, QChar('0'))
            << QString("Source: %1, ").arg(trig.trigSrc)
            << QString("Open: %1, ").arg(trig.trigOpen)
            << QString("Timer freq: %1, ").arg(trig.trigTimerFreq)
            << QString("Random freq: %1, ").arg(trig.trigRandomFreq)
            << QString("Random dead time: %1").arg(trig.trigRandomDeadTimeUs)
            << ")";
    return stream;
}

QDataStream &operator<<(QDataStream &out, const RcTrigConfig &r)
{
    out
            << r.xOffMask
            << r.trigSrc
            << r.trigOpen
            << r.trigTimerFreq
            << r.trigRandomFreq
            << r.trigRandomDeadTimeUs
               ;
    return out;
}

QDataStream &operator>>(QDataStream &in, RcTrigConfig &r)
{
    in
            >> r.xOffMask
            >> r.trigSrc
            >> r.trigOpen
            >> r.trigTimerFreq
            >> r.trigRandomFreq
            >> r.trigRandomDeadTimeUs
            ;
    return in;
}
