#include "RcTrigModule.h"

#include "regio/regop.h"

#include "RcTrigConfig.h"

enum {
    REG32_PULSER_FIX    = 0x0A,
    REG32_PULSER_RAND   = 0x0C,
    REG32_PULSER_DEAD   = 0x0E,
};

RcTrigModule::RcTrigModule(const MregBusInterface &bus)
    : BaseDeviceModule(bus)
{
}

QList<AFI_SDB_ID> RcTrigModule::getSupportedSdbIds()
{
    return {};
}

bool RcTrigModule::writeConfig(const RcTrigConfig &cfg)
{
    RegOpVector r;
    r << setTrigTimerFreqCmd(cfg.trigTimerFreq);
    r << setTrigRandomFreqCmd(cfg.trigRandomFreq);
    r << setTrigRandomDeadTimeCmd(cfg.trigRandomDeadTimeUs);
    return regOpExecRebased(r);
}

RegOpVector RcTrigModule::setTrigTimerFreqCmd(double freq)
{
    double periodInClocks = devClockFreq / freq;
    if (periodInClocks < 1) periodInClocks = 1;
    if (periodInClocks > quint32(-1)) periodInClocks = quint32(-1);
    quint32 regValue = periodInClocks;
    RegOpVector r;
    r.RegWrite32(REG32_PULSER_FIX, regValue);
    return r;
}

RegOpVector RcTrigModule::setTrigRandomFreqCmd(double freq)
{
    double period = 1.0 * freq * (quint32)(-1) / devClockFreq;
    if (period < 1) period = 1;
    if (period > quint32(-1)) period = quint32(-1);
    quint32 regValue = period;
    RegOpVector r;
    r.RegWrite32(REG32_PULSER_RAND, regValue);
    return r;
}

RegOpVector RcTrigModule::setTrigRandomDeadTimeCmd(double microseconds)
{
    double ticks = 1e-6 * microseconds * devClockFreq;
    if (ticks < 0) ticks = 0;
    if (ticks > quint32(-1)) ticks = quint32(-1);
    quint32 regValue = ticks;
    RegOpVector r;
    r.RegWrite32(REG32_PULSER_DEAD, regValue);
    return r;
}
