/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "RunLogicConfigConverter.h"

#include "RunLogicConfig.h"

namespace {
constexpr const char *k_timeLimitEnable = "timeLimitEnable";
constexpr const char *k_eventLimitEnable = "eventLimitEnable";
constexpr const char *k_timeLimit = "timeLimit";
constexpr const char *k_eventLimit = "eventLimit";
}

template<>
bool AbstractConfigConverter<RunLogicConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<RunLogicConfig>::toJsonObject(const RunLogicConfig &in)
{
    QVariantMap m;
    m[k_timeLimitEnable] = QVariant::fromValue(in.timeLimitEnable);
    m[k_eventLimitEnable] = QVariant::fromValue(in.eventLimitEnable);
    m[k_timeLimit] = QVariant::fromValue(static_cast<int>(in.timeLimit));
    m[k_eventLimit] = QVariant::fromValue(static_cast<int>(in.eventLimit));
    return fromVariantMap(m);
}

template<>
RunLogicConfig AbstractConfigConverter<RunLogicConfig>::fromJsonObject(const QJsonObject &in)
{
    RunLogicConfig out;
    update_value(in, k_timeLimitEnable, &out.timeLimitEnable);
    update_value(in, k_eventLimitEnable, &out.eventLimitEnable);
    update_value(in, k_timeLimit, &out.timeLimit);
    update_value(in, k_eventLimit, &out.eventLimit);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<RunLogicConfig>::toJsonValue(const RunLogicConfig &in)
{
    return toJsonObject(in);
}

template<>
RunLogicConfig AbstractConfigConverter<RunLogicConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<RunLogicConfig>::registerSubTypes()
{
}
