#include "RunStatisticModule.h"

#include "base/DeviceId.h"
#include "regio/regop.h"
#include "sdb/AfiSdbId.h"

#include "RunStatistic.h"

namespace {
inline constexpr const uint16_t REG64_COUNTERS_BASE = 0xC0;
inline constexpr const uint16_t REG64_TIMERS_BASE = 0x80;

enum {
    REG32_ID = 0x00,
    REG_CONTROL = 0x02,
    REG_LOCK = 0x03,
    REG32_TIMER_CLOCK_HZ = 0x04,
};

const quint16 REG_CONTROL_BIT_CLEAR_COUNTERS = 0x0001;
const quint16 REG_LOCK_BIT_LOCK = 0x0001;

} // namespace

RunStatisticModule::RunStatisticModule(const MregBusInterface &bus)
    : BaseDeviceModule(bus)
{
    if (!isAvailable()) {
        if (getDeviceId() == DEVICE_ID_PHOS_FEC) {
            setBaseAddr(0x700, 0xFF);
        }
    }
}

QList<AFI_SDB_ID> RunStatisticModule::getSupportedSdbIds()
{
    return {AFI_SDB_TDC_RUN_STAT_CNTS};
}

bool RunStatisticModule::resetCounters()
{
    RegOpVector r;
    r.RegWrite16(REG_CONTROL, REG_CONTROL_BIT_CLEAR_COUNTERS);
    r.RegWrite16(REG_CONTROL, 0);
    return regOpExecRebased(r);
}

bool RunStatisticModule::readRunStatistic(RunStatistic *result)
{
    return readStatus(result);
}

bool RunStatisticModule::readStatus(RunStatistic *result)
{
    Q_ASSERT(result);
    if (!result)
        return {};
    RegOpVector r;
    r.RegRead32(REG32_ID, &result->id);
    r.RegRead32(REG32_TIMER_CLOCK_HZ, &result->timerClock);
    r.RegWrite16(REG_LOCK, 0); // update counters
    r.RegWrite16(REG_LOCK, REG_LOCK_BIT_LOCK); // lock counters
    for (size_t i=0; i<result->counters.size(); i++)
        r.RegRead64(REG64_COUNTERS_BASE + i*4, &result->counters[i]);
    for (size_t i=0; i<result->timers.size(); i++)
        r.RegRead64(REG64_TIMERS_BASE + i*4, &result->timers[i]);
    r.RegWrite16(REG_LOCK, 0); // update counters
    if (!regOpExecRebased(r))
        return {};
    return true;
}

std::optional<RunStatistic> RunStatisticModule::readStatus()
{
    RunStatistic status;
    if (!readStatus(&status))
        return {};
    return status;
}
