#include "TdcCoreConfigConverter.h"
#include "TdcCoreConfig.h"

namespace {
constexpr const char *k_readout_fifo_size = "readout_fifo_size";
constexpr const char *k_max_event_size = "max_event_size";
constexpr const char *k_dead_time = "dead_time";
constexpr const char *k_reject_readout_fifo_full = "reject_readout_fifo_full";
constexpr const char *k_enable_error_mark = "enable_error_mark";
constexpr const char *k_resolution = "resolution";
constexpr const char *k_enable_pair = "enable_pair";
constexpr const char *k_core_clock_source = "core_clock_source";
constexpr const char *k_enable_matching = "enable_matching";
} // namespace

template<>
bool AbstractConfigConverter<HptdcAdjustConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<HptdcAdjustConfig>::toJsonObject(const HptdcAdjustConfig &in)
{
    QVariantMap m;

    m[k_readout_fifo_size] = QVariant::fromValue(in.readout_fifo_size);
    m[k_max_event_size] = QVariant::fromValue(in.max_event_size);
    m[k_dead_time] = QVariant::fromValue(in.dead_time);
    m[k_reject_readout_fifo_full] = QVariant::fromValue(in.reject_readout_fifo_full);
    m[k_enable_error_mark] = QVariant::fromValue(in.enable_error_mark);
    m[k_resolution] = QVariant::fromValue((int)in.resolution);
    m[k_enable_pair] = QVariant::fromValue(in.enable_pair);
    m[k_core_clock_source] = QVariant::fromValue(in.core_clock_source);
    m[k_enable_matching] = QVariant::fromValue(in.enable_matching);

    return fromVariantMap(m);
}

template<>
HptdcAdjustConfig AbstractConfigConverter<HptdcAdjustConfig>::fromJsonObject(const QJsonObject &in)
{
    HptdcAdjustConfig out;

    update_value(in, k_readout_fifo_size, &out.readout_fifo_size);
    update_value(in, k_max_event_size, &out.max_event_size);
    update_value(in, k_dead_time, &out.dead_time);
    update_value(in, k_reject_readout_fifo_full, &out.reject_readout_fifo_full);
    update_value(in, k_enable_error_mark, &out.enable_error_mark);
    update_value(in, k_resolution, &out.resolution);
    update_value(in, k_enable_pair, &out.enable_pair);
    update_value(in, k_core_clock_source, &out.core_clock_source);
    update_value(in, k_enable_matching, &out.enable_matching);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<HptdcAdjustConfig>::toJsonValue(const HptdcAdjustConfig &in)
{
    return toJsonObject(in);
}

template<>
HptdcAdjustConfig AbstractConfigConverter<HptdcAdjustConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<HptdcAdjustConfig>::registerSubTypes()
{}
