#include "TqdcCoreConfigConverter.h"

#include "TqdcCore/TqdcCoreConfig.h"
#include "base/ConfigConverterUtil.h"

namespace {
constexpr const char *k_latency = "latency";
constexpr const char *k_matchWin = "matchWin";
constexpr const char *k_hptdcStandby = "hptdcStandby";
constexpr const char *k_invertInput = "invertInput";
constexpr const char *k_invert_thr_trig = "invert_thr_trig";
constexpr const char *k_invert_zs_thr = "invert_zs_thr";
constexpr const char *k_hptdcHdr = "hptdcHdr";
constexpr const char *k_hptdcTrl = "hptdcTrl";
constexpr const char *k_leadEdge = "leadEdge";
constexpr const char *k_trailEdge = "trailEdge";

constexpr const char *k_chTdcEn = "chTdcEn";
constexpr const char *k_chAdcEn = "chAdcEn";
constexpr const char *k_chTrigEn = "chTrigEn";
constexpr const char *k_chThr = "chThr";
constexpr const char *k_chDigThr = "chDigThr";
constexpr const char *k_chDigTrigEn = "chDigTrigEn";
constexpr const char *k_chDigTrigRising = "chDigTrigRising";

constexpr const char *k_chWfInvert = "chWfInvert";
constexpr const char *k_chInv = "chInv";
constexpr const char *k_chGain = "chGain";

constexpr const char *k_chDigZsEn = "chDigZsEn";
constexpr const char *k_chDigZsRising = "chDigZsRising";
constexpr const char *k_chDigZsThr = "chDigZsThr";
}

template<>
bool AbstractConfigConverter<TqdcCoreConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<TqdcCoreConfig>::toJsonObject(const TqdcCoreConfig &in)
{
    QVariantMap m;
    m[k_latency] = QVariant::fromValue(in.latency);
    m[k_matchWin] = QVariant::fromValue(in.matchWin);
    m[k_hptdcStandby] = QVariant::fromValue(in.hptdcStandby);
    m[k_invertInput] = QVariant::fromValue(in.invertInput);
    m[k_invert_thr_trig] = QVariant::fromValue(in.invert_thr_trig);
    m[k_invert_zs_thr] = QVariant::fromValue(in.invert_zs_thr);
    m[k_hptdcHdr] = QVariant::fromValue(in.hptdcHdr);
    m[k_hptdcTrl] = QVariant::fromValue(in.hptdcTrl);
    m[k_leadEdge] = QVariant::fromValue(in.leadEdge);
    m[k_trailEdge] = QVariant::fromValue(in.trailEdge);

    m[k_chTdcEn] = ConfigConverterUtil::toJsonValue(in.chTdcEn);
    m[k_chAdcEn] = ConfigConverterUtil::toJsonValue(in.chAdcEn);
    m[k_chWfInvert] = ConfigConverterUtil::toJsonValue(in.chWfInvert);
    m[k_chInv] = ConfigConverterUtil::toJsonValue(in.chInv);
    m[k_chThr] = ConfigConverterUtil::toJsonValue(in.chThr);
    m[k_chTrigEn] = ConfigConverterUtil::toJsonValue(in.chTrigEn);
    m[k_chDigTrigEn] = ConfigConverterUtil::toJsonValue(in.chDigTrigEn);
    m[k_chDigTrigRising] = ConfigConverterUtil::toJsonValue(in.chDigTrigRising);
    m[k_chDigThr] = ConfigConverterUtil::toJsonValue(in.chDigThr);
    m[k_chGain] = ConfigConverterUtil::toJsonValue(in.chGain);

    m[k_chDigZsEn] = ConfigConverterUtil::toJsonValue(in.chDigZsEn);
    m[k_chDigZsRising] = ConfigConverterUtil::toJsonValue(in.chDigZsRising);
    m[k_chDigZsThr] = ConfigConverterUtil::toJsonValue(in.chDigZsThr);

    return fromVariantMap(m);
}

template<>
TqdcCoreConfig AbstractConfigConverter<TqdcCoreConfig>::fromJsonObject(const QJsonObject &in)
{
    TqdcCoreConfig out;
    update_value(in, k_latency, &out.latency);
    update_value(in, k_matchWin, &out.matchWin);
    update_value(in, k_hptdcStandby, &out.hptdcStandby);
    update_value(in, k_invertInput, &out.invertInput);
    update_value(in, k_invert_thr_trig, &out.invert_thr_trig);
    update_value(in, k_invert_zs_thr, &out.invert_zs_thr);
    update_value(in, k_hptdcHdr, &out.hptdcHdr);
    update_value(in, k_leadEdge, &out.leadEdge);
    update_value(in, k_trailEdge, &out.trailEdge);

    out.chTdcEn = ConfigConverterUtil::toMap<int, bool>(in[k_chTdcEn]);
    out.chAdcEn = ConfigConverterUtil::toMap<int, bool>(in[k_chAdcEn]);
    out.chWfInvert = ConfigConverterUtil::toMap<int, bool>(in[k_chWfInvert]);
    out.chInv = ConfigConverterUtil::toMap<int, bool>(in[k_chInv]);
    out.chThr = ConfigConverterUtil::toMap<int, int>(in[k_chThr]);
    out.chTrigEn = ConfigConverterUtil::toMap<int, bool>(in[k_chTrigEn]);
    out.chDigTrigEn = ConfigConverterUtil::toMap<int, bool>(in[k_chDigTrigEn]);
    out.chDigTrigRising = ConfigConverterUtil::toMap<int, bool>(in[k_chDigTrigRising]);
    out.chDigThr = ConfigConverterUtil::toMap<int, int>(in[k_chDigThr]);
    out.chGain = ConfigConverterUtil::toMap<int, int>(in[k_chGain]);

    out.chDigZsEn = ConfigConverterUtil::toMap<int, bool>(in[k_chDigZsEn]);
    out.chDigZsRising = ConfigConverterUtil::toMap<int, bool>(in[k_chDigZsRising]);
    out.chDigZsThr = ConfigConverterUtil::toMap<int, int>(in[k_chDigZsThr]);

    return out;
}

template<>
QJsonValue AbstractConfigConverter<TqdcCoreConfig>::toJsonValue(const TqdcCoreConfig &in)
{
    return toJsonObject(in);
}

template<>
TqdcCoreConfig AbstractConfigConverter<TqdcCoreConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<TqdcCoreConfig>::registerSubTypes()
{
}
