#include "TtlIoConfig.h"

#include <QDataStream>
#include <QDebug>

static const QMap<TtlIoInput, QString> TtlIoInput_Names = {
    {TtlIoInput::Trigger, "Trigger"},
    {TtlIoInput::Busy, "Busy"},
    {TtlIoInput::ActiveLowBusy, "ActiveLowBusy"},
    {TtlIoInput::EventNumberReset, "EvNumReset"}
};

static const QMap<TtlIoInput, QString> TtlIoInput_PrettyNames = {
    {TtlIoInput::Trigger, "Trigger"},
    {TtlIoInput::Busy, "Busy"},
    {TtlIoInput::ActiveLowBusy, "Active-low busy"},
    {TtlIoInput::EventNumberReset, "Event number reset"}
};

static const QMap<TtlIoOutput, QString> TtlIoOutput_Names = {
    {TtlIoOutput::AcceptedTrigger, "AcceptedTrig"},
    {TtlIoOutput::BlockedTrigger, "BlockedTrig"},
    {TtlIoOutput::Busy, "Busy"},
    {TtlIoOutput::Pps, "Pps"},
    {TtlIoOutput::Pulse24ns, "Pulse24ns"},
    {TtlIoOutput::Pulse48ns, "Pulse48ns"},
    {TtlIoOutput::Logic0, "Logic0"},
    {TtlIoOutput::Logic1, "Logic1"}
};

static const QMap<TtlIoOutput, QString> TtlIoOutput_PrettyNames = {
    {TtlIoOutput::AcceptedTrigger, "Accepted trigger"},
    {TtlIoOutput::BlockedTrigger, "Blocked trigger"},
    {TtlIoOutput::Busy, "Busy"},
    {TtlIoOutput::Pps, "PPS (only for WR compatible boards)"},
    {TtlIoOutput::Pulse24ns, "Pulse, period 24 ns (p24ns) (only for WR compatible boards)"},
    {TtlIoOutput::Pulse48ns, "Pulse, period 48 ns (p48ns) (only for WR compatible boards)"},
    {TtlIoOutput::Logic0, "Logic 0"},
    {TtlIoOutput::Logic1, "Logic 1"}
};

static const QList<TtlIoInput> TtlIoInput_List = TtlIoInput_Names.keys();
static const QList<TtlIoOutput> TtlIoOutput_List = TtlIoOutput_Names.keys();


void reg_ttl_io_t::setInput(TtlIoInput i)
{
    bit.isOutput = 0;
    bit.cfg = static_cast<int>(i);
}

void reg_ttl_io_t::setOutput(TtlIoOutput o)
{
    bit.isOutput = 1;
    bit.cfg = static_cast<int>(o);
}

TtlIoInput reg_ttl_io_t::getInput() const
{
    Q_ASSERT(bit.isOutput == 0);
    return static_cast<TtlIoInput>(bit.cfg);
}

TtlIoOutput reg_ttl_io_t::getOutput() const
{
    Q_ASSERT(bit.isOutput == 1);
    return static_cast<TtlIoOutput>(bit.cfg);
}

QString reg_ttl_io_t::toString() const
{
    return bit.isOutput ? TtlIoConfig::getName(getOutput()) : TtlIoConfig::getName(getInput());
}

QDebug operator<<(QDebug dbg, const TtlIoInput &r)
{
    dbg << TtlIoConfig::getName(r);
    return dbg;
}

QDebug operator<<(QDebug dbg, const TtlIoOutput &r)
{
    dbg << TtlIoConfig::getName(r);
    return dbg;
}

QDebug operator<<(QDebug dbg, const reg_ttl_io_t &r)
{
    if(r.bit.isOutput) {
        auto t = static_cast<TtlIoOutput>(r.bit.cfg);
        dbg << "Output" << TtlIoConfig::getName(t);
    } else {
        auto t = static_cast<TtlIoInput>(r.bit.cfg);
        dbg << "Input" << TtlIoConfig::getName(t);
    }
    return dbg;
}

QDebug operator<<(QDebug dbg, const TtlIoConfig &r)
{
    dbg << "TtlIoConfig:";
     for(int i=0; i<r.ttl_regs.size(); ++i) {
         dbg << QString("ttlIo#%1:").arg(i) << r.ttl_regs[i];
    }
    return dbg;
}

QList<TtlIoInput> TtlIoConfig::getInputs()
{
    return TtlIoInput_List;
}

QList<TtlIoOutput> TtlIoConfig::getOutputs()
{
    return TtlIoOutput_List;
}

QString TtlIoConfig::getName(const TtlIoInput &type)
{
    return TtlIoInput_Names.value(type);
}

QString TtlIoConfig::getName(const TtlIoOutput &type)
{
    return TtlIoOutput_Names.value(type);
}

QString TtlIoConfig::getPrettyName(const TtlIoInput &type)
{
    if (TtlIoInput_Names.contains(type))
        return TtlIoInput_PrettyNames.value(type);
    return TtlIoInput_Names.value(type);
}

QString TtlIoConfig::getPrettyName(const TtlIoOutput &type)
{
    if (TtlIoOutput_PrettyNames.contains(type))
        return TtlIoOutput_PrettyNames.value(type);
    return TtlIoOutput_PrettyNames.value(type);
}

QDataStream &operator<<(QDataStream &out, const reg_ttl_io_t &r)
{
    out << r.all;
    return out;
}

QDataStream &operator>>(QDataStream &in, reg_ttl_io_t &r)
{
    in >> r.all;
    return in;
}

QDataStream &operator<<(QDataStream &out, const TtlIoConfig &r)
{
    out << r.ttl_regs;
    return out;
}

QDataStream &operator>>(QDataStream &in, TtlIoConfig &r)
{
    in >> r.ttl_regs;
    return in;
}
