/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "WaveBlcConfigConverter.h"

#include "WaveBlcConfig.h"


namespace {
constexpr const char *k_polarity = "polarity";
constexpr const char *k_mode = "mode";
constexpr const char *k_mafLength = "mafLength";
constexpr const char *k_mafRoundoff = "mafRoundoff";
constexpr const char *k_thr = "thr";
constexpr const char *k_staticOffset = "staticOffset";
constexpr const char *k_commonOffset = "commonOffset";
}

template<>
bool AbstractConfigConverter<WaveBlcConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<WaveBlcConfig>::toJsonObject(const WaveBlcConfig &in)
{
    QVariantMap m;
    m[k_polarity] = QVariant::fromValue(static_cast<int>(in.common.polarity));
    m[k_mode] = QVariant::fromValue(static_cast<int>(in.common.mode));
    m[k_mafLength] = QVariant::fromValue(in.common.mafLength);
    m[k_mafRoundoff] = QVariant::fromValue(in.common.mafRoundoff);
    m[k_thr] = QVariant::fromValue(in.common.thr);
    m[k_staticOffset] = QVariant::fromValue(in.common.staticOffset);
    m[k_commonOffset] = QVariant::fromValue(in.commonOffset);
    return fromVariantMap(m);
}

template<>
WaveBlcConfig AbstractConfigConverter<WaveBlcConfig>::fromJsonObject(const QJsonObject &in)
{
    WaveBlcConfig out;
    update_value(in, k_polarity, &out.common.polarity);
    update_value(in, k_mode, &out.common.mode);
    update_value(in, k_mafLength, &out.common.mafLength);
    update_value(in, k_mafRoundoff, &out.common.mafRoundoff);
    update_value(in, k_thr, &out.common.thr);
    update_value(in, k_staticOffset, &out.common.staticOffset);
    update_value(in, k_commonOffset, &out.commonOffset);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<WaveBlcConfig>::toJsonValue(const WaveBlcConfig &in)
{
    return toJsonObject(in);
}

template<>
WaveBlcConfig AbstractConfigConverter<WaveBlcConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<WaveBlcConfig>::registerSubTypes()
{
}
