/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/


#include "WaveRecConfig.h"
#include "WaveRecConfigConverter.h"

#include "AdcErc/AdcErcConfigConverter.h"
#include "base/ConfigConverterUtil.h"

namespace {
constexpr const char *k_adcPreWin = "adcPreWin";
constexpr const char *k_readCellNumber = "readCellNumber";
constexpr const char *k_latency = "latency";
constexpr const char *k_matchWin = "matchWin";
constexpr const char *k_invertInput = "invertInput";
constexpr const char *k_invert_thr_trig = "invert_thr_trig";
constexpr const char *k_invert_zs_thr = "invert_zs_thr";
constexpr const char *k_zsMode = "zsEn";
constexpr const char *k_sparse = "sparse";

constexpr const char *k_chAdcEn = "chAdcEn";
constexpr const char *k_chTrigEn = "chTrigEn";
constexpr const char *k_chThr = "chThr";
constexpr const char *k_chInv = "chInv";
}

template<>
bool AbstractConfigConverter<WaveRecConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<WaveRecConfig>::toJsonObject(const WaveRecConfig &in)
{
    QVariantMap m;
    m[k_adcPreWin] = QVariant::fromValue(in.adcPreWin);
    m[k_readCellNumber] = QVariant::fromValue(in.readCellNumber);
    m[k_latency] = QVariant::fromValue(in.latency);
    m[k_matchWin] = QVariant::fromValue(in.matchWin);
//    m[k_invertInput] = QVariant::fromValue(in.invertInput);
//    m[k_invert_thr_trig] = QVariant::fromValue(in.invert_thr_trig);
//    m[k_invert_zs_thr] = QVariant::fromValue(in.invert_zs_thr);
    m[k_zsMode] = QVariant::fromValue(static_cast<int>(in.zsMode));

    m[k_chAdcEn] = ConfigConverterUtil::toJsonValue(in.chAdcEn);
//    m[k_chInv] = ConfigConverterUtil::toJsonValue(in.chInv);
//    m[k_chThr] = ConfigConverterUtil::toJsonValue(in.chThr);

    m[k_sparse] = QVariant::fromValue(in.sparse);

    return fromVariantMap(m);
}

template<>
WaveRecConfig AbstractConfigConverter<WaveRecConfig>::fromJsonObject(const QJsonObject &in)
{
    WaveRecConfig out;
    update_value(in, k_adcPreWin, &out.adcPreWin);
    update_value(in, k_readCellNumber, &out.readCellNumber);
    update_value(in, k_latency, &out.latency);
    update_value(in, k_matchWin, &out.matchWin);
//    update_value(in, k_invertInput, &out.invertInput);
//    update_value(in, k_invert_thr_trig, &out.invert_thr_trig);
//    update_value(in, k_invert_zs_thr, &out.invert_zs_thr);
    update_value(in, k_zsMode, &out.zsMode);

    out.chAdcEn = ConfigConverterUtil::toMap<int, bool>(in[k_chAdcEn]);
//    out.chInv = ConfigConverterUtil::toMap<int, bool>(in[k_chInv]);
//    out.chThr = ConfigConverterUtil::toMap<int, int>(in[k_chThr]);

    update_value(in, k_sparse, &out.sparse);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<WaveRecConfig>::toJsonValue(const WaveRecConfig &in)
{
    return toJsonObject(in);
}

template<>
WaveRecConfig AbstractConfigConverter<WaveRecConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<WaveRecConfig>::registerSubTypes()
{
    AbstractConfigConverter<SparseParams>::registerTypes();
}
