/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "WaveTrigConfigConverter.h"

#include "WaveTrigConfig.h"


namespace {
constexpr const char *k_en = "en";
constexpr const char *k_polarity = "polarity";
constexpr const char *k_trigLevel = "trigLevel";

constexpr const char *k_common = "common";
}

template<>
bool AbstractConfigConverter<WaveTrigChannelConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<WaveTrigChannelConfig>::toJsonObject(const WaveTrigChannelConfig &in)
{
    QVariantMap m;
    m[k_en] = QVariant::fromValue(in.en);
    m[k_polarity] = QVariant::fromValue(static_cast<int>(in.edge));
    m[k_trigLevel] = QVariant::fromValue(in.trigLevel);
    return fromVariantMap(m);
}

template<>
WaveTrigChannelConfig AbstractConfigConverter<WaveTrigChannelConfig>::fromJsonObject(const QJsonObject &in)
{
    WaveTrigChannelConfig out;
    update_value(in, k_en, &out.en);
    update_value(in, k_polarity, &out.edge);
    update_value(in, k_trigLevel, &out.trigLevel);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<WaveTrigChannelConfig>::toJsonValue(const WaveTrigChannelConfig &in)
{
    return toJsonObject(in);
}

template<>
WaveTrigChannelConfig AbstractConfigConverter<WaveTrigChannelConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<WaveTrigChannelConfig>::registerSubTypes()
{
}




template<>
bool AbstractConfigConverter<WaveTrigConfig>::classIsObject()
{
    return true;
}

template<>
QJsonObject AbstractConfigConverter<WaveTrigConfig>::toJsonObject(const WaveTrigConfig &in)
{
    QVariantMap m;
    m[k_common] = QVariant::fromValue(in.common);
    return fromVariantMap(m);
}

template<>
WaveTrigConfig AbstractConfigConverter<WaveTrigConfig>::fromJsonObject(const QJsonObject &in)
{
    WaveTrigConfig out;
    update_value(in, k_common, &out.common);
    return out;
}

template<>
QJsonValue AbstractConfigConverter<WaveTrigConfig>::toJsonValue(const WaveTrigConfig &in)
{
    return toJsonObject(in);
}

template<>
WaveTrigConfig AbstractConfigConverter<WaveTrigConfig>::fromJsonValue(const QJsonValue &in)
{
    return fromJsonObject(in.toObject());
}

template<>
void AbstractConfigConverter<WaveTrigConfig>::registerSubTypes()
{
    AbstractConfigConverter<WaveTrigChannelConfig>::registerTypes();
}
