#ifndef DACCONVERTER_H
#define DACCONVERTER_H

#include <QtCore>

class DacConverter
{
public:
    quint16 dacValue(double mV, double dacShift) const
    {
        const double &vRef = getGainedVRef();
        int dacData = 1. * mV / vRef * getDacMaxVal() + dacShift;
        return std::clamp(dacData, 0, (int)getDacMaxVal());
    }
    quint16 dacValue(double mV) const { return dacValue(mV, getDacDefaultZero()); }

    double dac_mV(quint16 dacData) const
    {
        return getGainedVRef() * ((int) dacData - getDacDefaultZero()) / getDacMaxVal();
    }

    virtual double getDacDefaultZero() const = 0;
    double getOutRange() const { return getGainedVRef(); }
    double getOutMax() const { return getGainedVRef() + getOutShift(); }
    double getOutMin() const { return getOutShift(); }

    virtual bool getGain() const = 0;
    virtual quint16 getDacMaxVal() const = 0;

protected:
    virtual double getVRef() const = 0;

private:
    double getOutShift() const { return dac_mV(0); }
    inline double getGainedVRef() const { return getGain() ? getVRef() * 2 : (getVRef()); }
};

#endif // DACCONVERTER_H
