/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef CLOCKCONTROLWIDGET_H
#define CLOCKCONTROLWIDGET_H

#include <QWidget>
#include <optional>

#include "base/DeviceIndex.h"
#include "device-modules/ClockControl/ClockControlConfig.h"
#include "device-modules/ClockControl/ClockStatus.h"

struct DeviceIndexEnabledMap;

namespace Ui {
class ClockControlWidget;
}

class ClockControlWidget : public QWidget
{
    Q_OBJECT

public:
    explicit ClockControlWidget(QWidget *parent = nullptr);
    virtual ~ClockControlWidget() override;
    ClockControlConfig getConfig() const { return cfg; }

public slots:
    void setConfig(const ClockControlConfig &newConfig);
    void setClockControlBp(const DeviceIndex &index, const ClockBuildParams &bp);
    void setClockControlStatus(const DeviceIndex &index, const std::optional<ClockStatus> &st);
    void setDeviceList(const DeviceIndexEnabledMap &map);

private slots:
    void on_comboBoxClockSource_currentIndexChanged(int);
    void on_checkBoxPllBypass_toggled(bool bypass);

signals:
    void configChanged(const ClockControlConfig &config);

private:
    void updateControlGui();
    void updateComboBoxList();
    void updateStatus() const;
    std::optional<quint16> getBuildParams() const;

    Ui::ClockControlWidget *ui;
    ClockControlConfig cfg;

    DeviceIndexSet devSet;
    QMap<DeviceIndex, ClockBuildParams> devBp;
    QMap<DeviceIndex, ClockStatus> devStatus;
};

#endif // CLOCKCONTROLWIDGET_H
