/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef FIRFILTERWIDGET_H
#define FIRFILTERWIDGET_H

#include <QMap>
#include <QWidget>

#include "FirFilter/FirFilterBuildParams.h"
#include "FirFilter/FirFilterConfig.h"
#include "base/DeviceIndex.h"

namespace Ui {
class FirFilterWidget;
}

class FirDesignDialog;
struct DeviceIndexEnabledMap;

class FirFilterWidget : public QWidget
{
    Q_OBJECT

public:
    explicit FirFilterWidget(QWidget *parent = nullptr);
    ~FirFilterWidget();

signals:
    void firFilterConfigChanged(const FirFilterConfig &config);

public slots:
    void setFirFilterConfig(const FirFilterConfig &newConfig);
    void setFirFilterBp(const DeviceIndex &index, const FirFilterBuildParams &bp);
    void setDeviceList(const DeviceIndexEnabledMap &map);

private slots:

    void on_comboBoxMode_currentIndexChanged(int index);
    void on_spinBoxTruncate_valueChanged(int arg1);
    void on_pushButtonFirDesign_clicked();
    void on_pushButtonSetFir_clicked();

private:
    void restoreGuiFromConfig();
    quint16 getSizeLimit() const;

    Ui::FirFilterWidget *ui;
    FirDesignDialog * const firDesignDialog;
    FirFilterConfig config;
    DeviceIndexSet devSet;
    QMap<DeviceIndex, FirFilterBuildParams> devBp;
};

#endif // FIRFILTERWIDGET_H
