#include "HptdcAdjustDialog.h"
#include "ui_HptdcAdjustDialog.h"
#include "base/gui_utils.h"
#include "base/TextUtils.h"

#include <QComboBox>
#include <QDebug>
#include <cmath>

//const QList<QPair<QString, int>> LEAD_RES_VALUES = {
//    {"100 ps",    100},
//    {"200 ps",    200},
//    {"400 ps",    400},
//    {"800 ps",    800},
//    {"1.6 ns",   1600},
//    {"3.12 ns",  3120},
//    {"6.25 ns",  6250},
//    {"12.5 ns", 12500}
//};
//const QList<int> DLL_MODE_VALUES = {40, 80, 320};

constexpr static const std::initializer_list<std::pair<TdcResIndex, const char *>>
    TdcResIndexText {
        {TDC_RES_INDEX_25, "25 ps"},
        {TDC_RES_INDEX_100, "100 ps"},
        {TDC_RES_INDEX_200, "200 ps"},
        {TDC_RES_INDEX_400, "400 ps"},
        {TDC_RES_INDEX_800, "800 ps"},
        {TDC_RES_INDEX_1600, "1.6 ns"}
    };

constexpr static const std::initializer_list<std::pair<int, const char *>>
    DllModeText {
        {0, "40 MHz"},
        {1, "80 MHz"},
        {2, "320 MHz"}
    };

constexpr static const std::initializer_list<std::pair<int, const char *>>
    DllClockText {
        {0, "clock_40"},
        {1, "pll_clock_40"},
        {2, "pll_clock_160"},
        {3, "pll_clock_320"},
        {4, "aux_clock"}
    };

constexpr static const std::initializer_list<std::pair<int, const char *>>
    CoreClockText {
        {0, "clock_40"},
        {1, "pll_clock_80"},
        {2, "pll_clock_160"},
        {3, "aux_clock"}
    };

HptdcAdjustDialog::HptdcAdjustDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::HptdcAdjustDialog)
{
    ui->setupUi(this);
    qRegisterMetaType<HptdcAdjustConfig>();
    ui->checkBox_rc_compression->setChecked(true);
    for(int i=0; i<=0b111; ++i)
        ui->comboBox_readout_fifo->addItem(QString::number(std::pow(2, i+1)), i);


    ui->checkBox_pairing_mode->setChecked(false);

    for (auto it = TdcResIndexText.begin(); it != TdcResIndexText.end(); ++it)
        ui->comboBoxLeadRes->addItem(it->second, it->first);

    for (auto it = DllModeText.begin(); it != DllModeText.end(); ++it)
        ui->comboBox_dll_mode->addItem(it->second, it->first);

    for (auto it = DllClockText.begin(); it != DllClockText.end(); ++it)
        ui->comboBox_dll_clock->addItem(it->second, it->first);

    for (auto it = CoreClockText.begin(); it != CoreClockText.end(); ++it)
        ui->comboBox_core_clock_source->addItem(it->second, it->first);

    ui->checkBox_rc_compression->setEnabled(false);
    ui->comboBox_dll_mode->setEnabled(false);
    ui->comboBox_dll_clock->setEnabled(false);

    updateLabels();
}

HptdcAdjustDialog::~HptdcAdjustDialog()
{
    delete ui;
}

void HptdcAdjustDialog::updateLabels()
{
    QMap<QString, QVariant> map = {
        {"mode_rc_compression", adjust_set.mode_rc_compression()},
        {"mode_rc", adjust_set.mode_rc()},
        {"leading_resolution", adjust_set.leading_resolution()},
        {"dll_mode", adjust_set.dll_mode()},
        {"dll_clock", adjust_set.dll_clock()}
    };
    ui->labelInfo->setText(makeBasicHtmlTable(map));
    setComboBoxCurrentItem(ui->comboBox_dll_mode, adjust_set.dll_mode());
    setComboBoxCurrentItem(ui->comboBox_dll_clock, adjust_set.dll_clock());
    ui->checkBox_rc_compression->setChecked(adjust_set.mode_rc_compression());
}

void HptdcAdjustDialog::setConfig(const HptdcAdjustConfig &s)
{
    adjust_set = s;
    RecursiveWidgetSignalBlocker blocker(this);
    setComboBoxCurrentItem(ui->comboBox_readout_fifo, s.readout_fifo_size);
    setComboBoxCurrentItem(ui->comboBoxLeadRes, (int)s.resolution);
    ui->checkBox_pairing_mode->setChecked(s.enable_pair);
    ui->comboBox_core_clock_source->setCurrentIndex(s.core_clock_source);
    updateLabels();
}

void HptdcAdjustDialog::on_comboBox_readout_fifo_currentIndexChanged(int index)
{
    bool ok;
    int v = ui->comboBox_readout_fifo->itemData(index).toInt(&ok);
    Q_ASSERT(ok && index == v);
    adjust_set.readout_fifo_size = static_cast<quint8>(v);
    write_setup();
}

void HptdcAdjustDialog::on_comboBoxLeadRes_currentIndexChanged(int)
{
    adjust_set.resolution = static_cast<TdcResIndex>(ui->comboBoxLeadRes->currentData().toInt());
    updateLabels();
    write_setup();
}

void HptdcAdjustDialog::on_checkBox_pairing_mode_clicked(bool checked)
{
    adjust_set.enable_pair = checked;
    write_setup();
}

void HptdcAdjustDialog::on_comboBox_core_clock_source_currentIndexChanged(int index)
{
    adjust_set.core_clock_source = static_cast<quint16>(index);
    write_setup();
}

void HptdcAdjustDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    switch (ui->buttonBox->standardButton(button)) {
    case QDialogButtonBox::Close:
        hide();
        return;
    default:
        Q_UNREACHABLE();
    }
}

void HptdcAdjustDialog::write_setup()
{
    emit configChanged(adjust_set);
}

void HptdcAdjustDialog::activate_dialog()
{
    emit configChanged(adjust_set);
}
