#include "TtlIoWidget.h"
#include "ui_TtlIoWidget.h"

#include <QComboBox>
#include <QLabel>
#include <QSizePolicy>

#include <QDebug>

#include "base/Globals.h"

TtlIoWidget::TtlIoWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::TtlIoWidget)
{
    ui->setupUi(this);
}

TtlIoWidget::~TtlIoWidget()
{
    delete ui;
}

void TtlIoWidget::setStartUpCfg(const TtlIoConfig &sturtUpCfg)
{
    const auto &cnt = sturtUpCfg.ttl_regs.size();
    bool hasChanges = false;
    for(int i=singleTtlWidgets.size(); i<cnt ; ++i) {
        hasChanges = true;
        addTtl(sturtUpCfg.ttl_regs[i]);
    }
    if(hasChanges)
        emit ttlCfgChanged(cfg);
}

void TtlIoWidget::setTtlCfg(const TtlIoConfig &ttlCfg)
{
    if(cfg == ttlCfg)
        return;
    cfg = ttlCfg;
    const auto &cnt = cfg.ttl_regs.size();

    // Update existing
    for(int i=0; i<singleTtlWidgets.size(); ++i)
        singleTtlWidgets[i]->setRegConfig(cfg.ttl_regs[i]);

    // Add new
    for(int i=singleTtlWidgets.size(); i<cnt; ++i)
        addTtl(cfg.ttl_regs[i]);

    // Remove extra
    while(singleTtlWidgets.size()>cnt)
        removeTtl();
}

void TtlIoWidget::onFsmStateChange(const FsmState &state)
{
    bool en = state==FsmState::Ready || state==FsmState::Error
            || Globals::getDebugMode();
    setEnabled(en);
}

void TtlIoWidget::ttlItemChanged(int index)
{
    for(auto w : singleTtlWidgets) {
        if(w->cb == sender()) {
            Q_ASSERT(cfg.ttl_regs.size()>w->id);
            auto &ttlCfg = cfg.ttl_regs[w->id];
            auto cbData = w->cb->itemData(index);
            ttlCfg.all = cbData.toUInt();
            emit ttlCfgChanged(cfg);
            return;
        }
    }
}

void TtlIoWidget::addTtl(reg_ttl_io_t r)
{
    auto id = singleTtlWidgets.size();
    if(cfg.ttl_regs.size()<id+1) {
        cfg.ttl_regs.append(r);
    }
    const auto &regCfg = cfg.ttl_regs[id];
    auto w = new SingleTtlWidget(id, this);
    singleTtlWidgets.append(w);
    ui->gridLayout->addWidget(w->lab, id, 0, 1, 1);
    ui->gridLayout->addWidget(w->cb, id, 1, 1, 1);

    w->setRegConfig(regCfg);
    connect(w->cb, SIGNAL(currentIndexChanged(int)), this, SLOT(ttlItemChanged(int)));
}

void TtlIoWidget::removeTtl()
{
    auto w = singleTtlWidgets.takeLast();
    ui->gridLayout->removeWidget(w->lab);
    ui->gridLayout->removeWidget(w->cb);
    delete w;

    while(cfg.ttl_regs.size() > singleTtlWidgets.size())
        cfg.ttl_regs.removeLast();
}

SingleTtlWidget::SingleTtlWidget(int id, QWidget *parent)
    : QWidget(parent)
    ,id(id)
    ,lab(new QLabel(this))
    ,cb(new QComboBox(this))
{
    lab->setText(QString("LEMO #%1").arg(id+1));
    lab->setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);

    QSizePolicy sp = cb->sizePolicy();
    sp.setHorizontalPolicy(QSizePolicy::Fixed);
    cb->setSizePolicy(sp);
    QStringList cbToolTip;
    cbToolTip << "Inputs:";
    for(auto i :TtlIoConfig::getInputs()) {
        reg_ttl_io_t r;
        r.setInput(i);
        cb->addItem(TtlIoConfig::getName(i).prepend("In: "), r.all);
        cbToolTip << QString("%1 - %2").arg(TtlIoConfig::getName(i))
                   .arg(TtlIoConfig::getPrettyName(i));
    }
    cb->insertSeparator(cb->count());
    cbToolTip << "" << "Outputs:";
    for(auto i :TtlIoConfig::getOutputs()) {
        reg_ttl_io_t r;
        r.setOutput(i);
        cb->addItem(TtlIoConfig::getName(i).prepend("Out: "), r.all);
        cbToolTip << QString("%1 - %2").arg(TtlIoConfig::getName(i))
                   .arg(TtlIoConfig::getPrettyName(i));
    }
    cb->setToolTip(cbToolTip.join('\n'));
}

SingleTtlWidget::~SingleTtlWidget()
{
    delete cb;
    delete lab;
}

void SingleTtlWidget::setRegConfig(const reg_ttl_io_t &regCfg)
{
    const auto &targ = QVariant::fromValue(regCfg.all);
    int id = cb->findData(targ);
    if(id!=-1)
        cb->setCurrentIndex(id);
    else {
        for(int i=0; i<cb->count(); ++i) {
            qInfo() << i << cb->itemText(i) << cb->itemData(i);
        }
    }
}
