#ifndef FILEREADERDIALOG_H
#define FILEREADERDIALOG_H

#include <QDialog>

class FileReader;
class QSemaphore;

namespace Ui {
class FileReaderDialog;
}

#include "FileReaderConfig.h"

class FileReaderDialog : public QDialog
{
    Q_OBJECT

public:
    explicit FileReaderDialog(QSemaphore *freeRawDataSem, QWidget *parent = nullptr);
    ~FileReaderDialog() override;
    void configUpdated();
    void setPackSizeLimit(qint64 limit);
    void reopenLastFile();

public slots:
    void setFileReaderConfig(const FileReaderConfig &cfg);

signals:
    void fileNameUpdated(QString fileName);
    void changeRecord(int recordNum, int count=1);
    void dataAccuired(QByteArray);
    void closeAll();
    void closeFile();
    void fileReaderConfigChanged(const FileReaderConfig &cfg);
    void packSizeLimitUpdated(qint64 limit);

private slots:
    void on_pushButtonSetFile_clicked();
    void on_spinBoxCurEvent_valueChanged(int evNum);
    void on_pushButtonBackward_toggled(bool checked);
    void on_pushButtonStop_toggled(bool checked);
    void on_pushButtonForward_toggled(bool checked);
    void on_spinBoxInterval_valueChanged(int intervalMs);
    void on_comboBoxReadBy_currentIndexChanged(int);

    void playback();
    void fileMapUpdated(qint64 fileSize, QString lastModify, int recordsSize, double decodedPart);
    void eventRead(quint32 ev_first, quint32 ev_last, qint64 evSize);

public slots:
    void onCloseFile();

private:
    void saveSettings();
    void updateProgressBar(int recordNum) const;

    Ui::FileReaderDialog *ui;
    FileReaderConfig config;
    QSemaphore * const freeRawDataSem;
    FileReader * const reader;
    QThread * const fileReaderThread;
    QTimer * const playbackTimer;
    bool playbackForward;
    bool block_write_settings;
};

#endif // FILEREADERDIALOG_H
