#include "BaseAppCore.h"

#include <QDebug>
#include <QLoggingCategory>

#include "app/AppVersion.h"
#include "base/ConfigIndexName.h"
#include "base/Globals.h"
#include "base/ThreadController.h"
#include "metric-sender/MetricSender.h"
#include "remote-control-server/RemoteControlServer.h"

namespace {
Q_DECLARE_LOGGING_CATEGORY(lc_appcore)
Q_LOGGING_CATEGORY(lc_appcore, "afi.appcore")
}

BaseAppCore::BaseAppCore(QObject *parent)
    : QObject{parent}
    , remoteControlServer(new RemoteControlServer(this))
{
    new ThreadController(remoteControlServer, this);

    connect(this, &BaseAppCore::sendRunError, this, &BaseAppCore::onRunError);
    connect(this, &BaseAppCore::sendRunWarning, this, &BaseAppCore::onRunWarning);
}

BaseAppCore::~BaseAppCore()
{}

void BaseAppCore::configSelectorChanged(const ConfigIndexName &conf)
{
    programIndex = conf.getIndex();
    if(progDescr.getProgramIndex() != programIndex) {
        auto old = progDescr;
        progDescr.setClientIndex({Globals::getDaqModule(), programIndex});
        progDescr.uuid = QUuid::createUuid();
        emit closeProgram(old);
    }
    progDescr.ver_date = getAppDateStr();
    progDescr.ver_hash = getAppVersion();
    emit updateProgramDescription(progDescr);
    MetricSender::setBaseTag("index", conf.getIndex().toString());

    ClientIndex ci{Globals::getDaqModule(), conf.getIndex()};
    if(rcProgramState.index != ci) {
        rcProgramState.index = ci;
        emit rcProgramStateChanged(rcProgramState);
    }
}

void BaseAppCore::onRunError(const RcRunId &runId, QString text)
{
    qWarning(lc_appcore).noquote() << runId << text;
}

void BaseAppCore::onRunWarning(const RcRunId &runId, QString text)
{
    qCritical(lc_appcore).noquote() << runId << text;
}

void BaseAppCore::remoteStatusUpdated(const RemoteControlServerStatus &status)
{
    bool changed = progDescr.updateProgramInterface(
        ProgramInterfaceRemoteControl, 0,
        status.tcpPort, status.listening, !status.portInUse,
        status.peerHost, status.peerPort);
    if(changed)
        emit updateProgramDescription(progDescr);

    if(!status.portInUse && underRemoteCtrl)
        qWarning() << "Remote control disconnected.";
    underRemoteCtrl = status.portInUse;
}

void BaseAppCore::onFsmStateChange(const FsmState &state, const FsmState &)
{
    if(rcProgramState.fsm != state) {
        rcProgramState.fsm = state;
        emit rcProgramStateChanged(rcProgramState);
    }
}
