/*
**    Copyright 2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef READOUTAPPSIGNALROUTER_H
#define READOUTAPPSIGNALROUTER_H

#include "base/DeviceIndex.h"
#include "BaseSignalRouter.h"

class AdcMpdTrigDialog;
class AdcSerDesCalDialog;
class AsyncDeviceCommander;
class BaseConfigDispatcher;
class CalConfigDb;
class ClockControlDialog;
class CrosspointSwitchDilog;
class DeviceDispatcher;
class DeviceMgrFsm;
class DeviceStatusDialog;
class DeviceStatusTableViewFancy;
class DiscoverDialog;
class EvNumChecker;
class FeLinkStatusDialog;
class HptdcStatusDialog;
class ModularMregDevice;
class MstreamMregDevGlobal;
class MStreamStatisticDialog;
class NetworkPortConfigDialog;
class SparseDialog;
class ReadoutAppCore;
class ReadoutAppMainWindow;

class ReadoutAppSignalRouter : public BaseSignalRouter
{
public:
    explicit ReadoutAppSignalRouter() = default;
    virtual ~ReadoutAppSignalRouter() = default;
    void setDeviceIdFilter(const DeviceIdSet &filter);

    void setAdcMpdTrigDialog(AdcMpdTrigDialog *p)
    { adcMpdTrigDialog = p; }
    void setAdcSerDesCalDialog(AdcSerDesCalDialog *p)
    { adcSerDesCalDialog = p; }
    void setCalConfigDb(CalConfigDb *p)
    { calConfigDb = p; }
    void setConfigDispatcher(BaseConfigDispatcher *p)
    { configDispatcher = p;}
    void setClockControlDialog(ClockControlDialog *p)
    { clockControlDialog = p;}
    void setCrosspointSwitchConfigDialog(CrosspointSwitchDilog *p)
    { crosspointSwitchDilog = p;}
    void setDeviceCommander(AsyncDeviceCommander *p)
    { deviceCommander = p;}
    void setDeviceDispatcher(DeviceDispatcher *p)
    { deviceDispatcher = p;}
    void setDeviceStatusDialog(DeviceStatusDialog *p)
    { deviceStatusDialog = p; }
    void setDeviceStatusTableView(DeviceStatusTableViewFancy *p)
    { deviceStatusTableView = p; }
    void setDiscoverDialog(DiscoverDialog *p)
    { discoverDialog = p;}
    void setEvNumChecker(EvNumChecker *p)
    { evNumChecker = p; }
    void setFeLinkStatusDialog(FeLinkStatusDialog *p)
    { feLinkStatusDialog = p; }
    void setDeviceMgrFsm(DeviceMgrFsm *p);
    void setHptdcStatusDialog(HptdcStatusDialog *p)
    { hptdcStatusDialog = p; }
    void setMStreamStatisticDialog(MStreamStatisticDialog *p)
    { mStreamStatisticDialog = p; }
    void setNetworkPortConfigDialog(NetworkPortConfigDialog *p)
    { networkPortConfigDialog = p; }
    void setSparseDialog(SparseDialog *p)
    { sparseDialog = p; }
    void setReadoutAppCore(ReadoutAppCore *p);
    void setReadoutAppMainWindow(ReadoutAppMainWindow *p);

public:
    virtual void connectAll() const override;
    virtual void connect_ConfigurationDispatcher() const;
    virtual void connect_Core() const;
    virtual void connect_FSM() const override;
    virtual void connect_device(const MstreamMregDevGlobal *devGlob, const ModularMregDevice *dev) const;
    virtual void connect_deviceCommander() const;
    virtual void connect_deviceDispatcher() const;
    virtual void connect_EvNumChecker() const;
    virtual void connect_MainWindow() const override;
    virtual void connect_RemoteControlServer() const override;

protected:
    DeviceIdSet deviceIdFilter;

    BaseConfigDispatcher *configDispatcher = nullptr;
    CalConfigDb * calConfigDb = nullptr;
    AsyncDeviceCommander * deviceCommander = nullptr;
    DeviceDispatcher * deviceDispatcher = nullptr;
    DeviceMgrFsm * deviceMgrFsm = nullptr;
    ReadoutAppCore *readoutAppCore = nullptr;

#ifdef QT_WIDGETS_LIB
    AdcMpdTrigDialog *adcMpdTrigDialog = nullptr;
    AdcSerDesCalDialog *adcSerDesCalDialog = nullptr;
    CrosspointSwitchDilog *crosspointSwitchDilog = nullptr;
    ClockControlDialog *clockControlDialog = nullptr;
    DeviceStatusDialog * deviceStatusDialog = nullptr;
    DeviceStatusTableViewFancy *deviceStatusTableView = nullptr;
    DiscoverDialog * discoverDialog = nullptr;
    EvNumChecker *evNumChecker = nullptr;
    FeLinkStatusDialog *feLinkStatusDialog = nullptr;
    HptdcStatusDialog *hptdcStatusDialog = nullptr;
    MStreamStatisticDialog *mStreamStatisticDialog = nullptr;
    NetworkPortConfigDialog *networkPortConfigDialog = nullptr;
    SparseDialog *sparseDialog = nullptr;
    ReadoutAppMainWindow *readoutAppMainWindow = nullptr;
#endif
};

#endif // READOUTAPPSIGNALROUTER_H
