#ifndef MSTREAMSTATISTICWIDGET_H
#define MSTREAMSTATISTICWIDGET_H

#include <QWidget>
#include "base/DeviceIndexEnabled.h"
#include "mstream-lib/MStreamDump.h"

namespace Ui {
class MStreamStatisticWidget;
}

class MStreamStatisticWidget : public QWidget
{
    Q_OBJECT

public:
    explicit MStreamStatisticWidget(QWidget *parent = nullptr);
    ~MStreamStatisticWidget() override;
    QMap<DeviceIndex, MStreamStat> getStat() const { return prev_ms_stat; }

public slots:
    void setDeviceList(const DeviceIndexEnabledMap &new_devices);
    void stat_updated(const MStreamStat &ms_stat);
    void stat_from_receiver_updated(const ReceiverStat &stat);
    void on_checkBox_toggled(bool checked);
    void set_run_state(bool run);

signals:
    void deviceListChanged(const DeviceIndexEnabledMap &);

private:
    void add_device(DeviceIndex index);
    void remove_device(DeviceIndex index);
    void remove_all_devices();
    void init_table();
    int index_to_row(DeviceIndex index) const;
    DeviceIndex row_to_index(int row) const;
    void update_column_visible() const;
    void set_table_text(int row, int col, quint64 val);
    void set_table_text(int row, int col, quint32 val);
    void set_table_text(int row, int col, double val);

    Ui::MStreamStatisticWidget *ui;
    DeviceIndexSet devices;
    QMap<DeviceIndex, MStreamStat> prev_ms_stat;

    bool show_all;
    bool run;
};
#endif // MSTREAMSTATISTICWIDGET_H
