/*
**    Copyright 2021 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "MessagePublisher.h"

#include <QSettings>
#include <QUrl>

#include "base/Globals.h"
#include "base/singleton.h"
#include "impl/AbstractPublisher.h"
#include "impl/DummyPublisher.h"
#ifdef MQTT_PUBLISHER
#include "impl/mqtt/MqttPublisher.h"
#endif
#ifdef REDIS_PUBLISHER
#include "impl/redis/RedisPublisher.h"
#endif
#ifdef ZMQKV_PUBLISHER
#include "impl/zmqkv/ZmqKvPublisher.h"
#endif

MessagePublisher::MessagePublisher(QObject *parent) : QObject(parent)
{
    setObjectName("MessagePublisher");
    // impl_dummy = new DummyPublisher(this);
#ifdef MQTT_PUBLISHER
    impl_mqtt = new MqttPublisher("mqtt", this);
#endif
#ifdef REDIS_PUBLISHER
    impl_redis = new RedisPublisher("redis", this);
#endif
#ifdef ZMQKV_PUBLISHER
    impl_zmqkv = new ZmqKvPublisher("zmqkv", this);
#endif
}

MessagePublisher &MessagePublisher::instance()
{
    MessagePublisher &instance = Singleton<MessagePublisher>::instance();
    return instance;
}

void MessagePublisher::pub(const QString &key, const QByteArray &data)
{
    if (impl_dummy && !impl_mqtt && !impl_redis && !impl_zmqkv)
        impl_dummy->pub(key, data);
    if (impl_mqtt)
        impl_mqtt->pub(key, data);
    if (impl_redis)
        impl_redis->pub(key, data);
    if (impl_zmqkv)
        impl_zmqkv->pub(key, data);
}
