#include "PrintConfigMap.h"
#include "base/DaqModule.h"
#include "base/ConfigSelector.h"
#include "base/Globals.h"
#include "BaseConfig.h"

const QString col_program_index = "Program index";
const QString col_configuration_name = "Configuration name";
const QString col_datetime = "DateTime";

void printConfigMap(){
    const DaqModule &module = Globals::getDaqModule();
    QMap<ProgramIndex, QMap<ConfigurationName, QDateTime> > config_map =
        BaseConfig().get_config_map(module);

    QList<QStringList> tableData; // item = tableData[col][row];
    tableData << QStringList(col_program_index);
    tableData << QStringList(col_configuration_name);
    tableData << QStringList(col_datetime);

    for(auto it_pi=config_map.begin(); it_pi!=config_map.end(); ++it_pi) {
        auto &program_index = it_pi.key();
        for(auto it_cn=it_pi->begin(); it_cn!=it_pi->end(); ++it_cn) {
            tableData[0] <<  program_index.getIndex();
            tableData[1] <<  it_cn.key();
            tableData[2] <<  it_cn.value().toString(Qt::ISODate);
        }
    }

    const auto typeName = module.toString();
    if(tableData[0].size()==1) {
        fprintf(stdout, "\nNo saved %s configurations.",
                typeName.toStdString().c_str());
        exit(0);
    }

    QMap<int, int> columnsSize;
    for(int col=0; col<tableData.size(); ++col){
        for(auto &str : tableData[col])
            columnsSize[col] = qMax(columnsSize.value(col), str.size());
    }

    QString  separator;
    for(int colSize : columnsSize)
        separator += QString("+%1").arg("", colSize+2, '-');
    separator += "+\n";

    QString allText = QString("\nList of saved %1 configurations: \n").arg(typeName);
    QString prev_program_index;
    for(int row=0; row<tableData[0].size(); ++row) {
        if(row == 0 || row == 1 ){
            allText += separator;
            prev_program_index = tableData[0][row];
        }
        else{
            if(tableData[0][row] != prev_program_index){
                prev_program_index = tableData[0][row];
                allText += separator;
            }
        }
        QString rowStr;
        for(int col=0; col<columnsSize.size(); ++col) {
            const auto &item = tableData[col][row];
            rowStr += QString("| %1%2%3%4")
                    .arg(row?"":"\033[91m")
                    .arg(item)
                    .arg(row?"":"\033[0m")
                    .arg("", columnsSize[col]-item.size()+1);
        }
        rowStr += "|\n";
        allText += rowStr;

        qInfo() << "tableData[0][row] " << tableData[0][row] << " program_index " << prev_program_index;
    }

    allText += separator;
    fprintf(stdout, "%s", allText.toStdString().c_str());
    exit(0);
}
