/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "RootConfigUtil.h"

#include <QDateTime>
#include <QJsonObject>
#include <QVariant>

#include "MongoConsts.h"
#include "base/ConfigMetaConverter.h"
#include "base/ConfigDocumentConverter.h"

namespace {
QDateTime getTimestampRecord(const QJsonObject &obj, QString key)
{
    auto del_date_val = obj[key];

    if(del_date_val.isString())
        return QDateTime::fromString(del_date_val.toString(), Qt::ISODate);

    if(del_date_val.isObject()){
        QVariant value = del_date_val.toObject()["$date"].toVariant();
        return QDateTime::fromMSecsSinceEpoch(value.toLongLong());
    }

    return QDateTime();
}

void updateTimestampRecord(QJsonObject *obj, QString key, const QDateTime &time)
{
    (*obj)[key] = time.toString(Qt::ISODate);
}
} //namespace

void RootConfigUtil::setModificationTimestamp(QJsonObject *meta)
{
    updateTimestampRecord(meta, k_date_of_modification, QDateTime::currentDateTime());
}

void RootConfigUtil::setDeletionTimestamp(QJsonObject *obj)
{
    updateTimestampRecord(obj, k_delete_datetime, QDateTime::currentDateTime());
}

QDateTime RootConfigUtil::getModificationTimestamp(const QJsonObject &cfg_record)
{
    return getTimestampRecord(cfg_record[k_meta].toObject(), k_date_of_modification);
}

bool RootConfigUtil::is_phantom(const QJsonObject &cfg_db_record)
{
    return cfg_db_record[k_meta].toObject()[k_phantom].toBool(true);
}
