/*
**    Copyright 2011-2022 Ivan Filippov
**    Copyright 2011-2022 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#ifndef BASEFRAGMENT_H
#define BASEFRAGMENT_H

#include <QByteArray>
#include <QMap>

struct MStreamFragmentHeader;

struct BaseFragment
{
    BaseFragment() { clear(); }
    quint16 fragmentId;
    bool free;
    bool completed;
    bool closed;

    quint8 deviceId;
    quint32 deviceSerial;
    quint8 subType;
    quint32 evNum;

    QByteArray data; // <offset(quint32:24), data>
    QMap<int, int> parts;
    uint32_t expectedLen; // shift+size of last fragment
    uint32_t curLen; // sum of continious parts start from 0

    void clear(){
        free=true;
        completed=false;
        closed=false;

        deviceId=0;
        deviceSerial=0;
        subType=0;
        evNum=0;

        data.resize(0);
        parts.clear();
        expectedLen=0;
        curLen=0;
    }
    void init(const MStreamFragmentHeader *f);
    void insert(const MStreamFragmentHeader *f);
};

#endif // BASEFRAGMENT_H
