#ifndef QXW_RANDGEN_H
#define QXW_RANDGEN_H

#include <cmath>

namespace qxw {

    // An unnormalized gaussian function with mean=0 and sigma
    // Divide result by sqrt(2*Pi) = 2.50662827463100024 if normalized output needed
    double gauss (double x, double sigma);

    //  Machine independent random number generator.
    //  Produces uniformly-distributed floating points in [0,1]
    //  Identical sequence on all machines of >= 32 bits.
    //  Periodicity > 10**14
    double rndm ();

    // returns a uniform deviate on the interval ]x1, x2].
    double ranuni (double x1, double x2);

    //      Return a number distributed following a gaussian with sigma
    double rangaus (double sigma);

    // Simulate detector pulse
    // amplitude a, time t, risetime rt, falltime ft
    double detpulse (double a, double t, double rt, double ft);
}

#endif                          /* QXW_RANDGEN_H */
