/*
**    Copyright 2020 Ilia Slepnev
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/

#include "RemoteControlProtocol.h"

#include <QTcpSocket>
#include <QJsonDocument>

void send_rc_block(QTcpSocket *tcpSocket, quint32 type, const QByteArray &payload)
{
    if (tcpSocket == nullptr || !tcpSocket->isValid()
            || (tcpSocket->state() != QTcpSocket::ConnectedState)) {
        return;
    }

    TypeLengthHeader hdr;
    hdr.type = type;
    hdr.len = static_cast<quint32>(payload.size());

    QByteArray data = QByteArray::fromRawData(reinterpret_cast<const char *>(&hdr), sizeof(TypeLengthHeader));
    data.append(payload);

    int written = static_cast<int>(tcpSocket->write(data));
    if (written != data.size()) {
        qWarning("RemoteControlProtocol failed to send a block (type:0x%x)",
                 type);
    }
}

void send_rc_command(QTcpSocket *tcpSocket, const QStringList &commandList)
{
    auto str = commandList.join(' ').append(RC_CMD_END);
    send_rc_block(tcpSocket, RC_MSG_CMD_V4, str.toUtf8());
}

void send_rc_json(QTcpSocket *tcpSocket, const QJsonObject &obj)
{
    send_rc_block(tcpSocket, RC_MSG_JSON_V5, QJsonDocument(obj).toJson(QJsonDocument::Compact));
}

void send_run_document(QTcpSocket *tcpSocket, const QByteArray &payload)
{
    send_rc_block(tcpSocket, RC_MSG_CONFIG_V4, payload);
}
